@@ -1,20 +1,19 @@
 def __rope_start_everything():
     import os
-    import sys
     import socket
+    import sys
 
     try:
         import cPickle as pickle
     except ImportError:
         import pickle
-    import marshal
-    import inspect
-    import types
-    import threading
-    import rope.base.utils.pycompat as pycompat
     import base64
     import hashlib
     import hmac
+    import inspect
+    import marshal
+    import threading
+    import types
 
     class _MessageSender:
         def send_data(self, data):
@@ -145,7 +144,7 @@ def __rope_start_everything():
                 return ("unknown",)
 
         def _get_persisted_builtin(self, object_):
-            if isinstance(object_, pycompat.string_types):
+            if isinstance(object_, str):
                 return ("builtin", "str")
             if isinstance(object_, list):
                 holding = None
@@ -194,7 +193,7 @@ def __rope_start_everything():
                 return self._get_persisted_code(object_.__func__.__code__)
             if isinstance(object_, types.ModuleType):
                 return self._get_persisted_module(object_)
-            if isinstance(object_, pycompat.string_types + (list, dict, tuple, set)):
+            if isinstance(object_, (str, list, dict, tuple, set)):
                 return self._get_persisted_builtin(object_)
             if isinstance(object_, type):
                 return self._get_persisted_class(object_)
@@ -231,7 +230,9 @@ def __rope_start_everything():
     if send_info != "-":
         data_sender = _FunctionCallDataSender(send_info, project_root)
     del sys.argv[1:4]
-    pycompat.execfile(file_to_run, run_globals)
+    with open(file_to_run) as f:
+        code = compile(f.read(), file_to_run, "exec")
+        exec(code, run_globals)
     if send_info != "-":
         data_sender.close()
 
