@@ -1,9 +1,9 @@
 # Based on super lightweight Simple Top-Down Parser from http://effbot.org/zone/simple-top-down-parsing.htm
 # and https://bitbucket.org/emacsway/sqlbuilder/src/default/sqlbuilder/smartsql/contrib/evaluate.py
 import re
-from rope.base.utils import pycompat
-from rope.base.oi.type_hinting import utils
+
 from rope.base import utils as base_utils
+from rope.base.oi.type_hinting import utils
 
 
 class SymbolBase:
@@ -27,7 +27,7 @@ class SymbolBase:
 
     def __repr__(self):
         if self.name == "(name)":
-            return "({} {})".format(self.name[1:-1], self.value)
+            return f"({self.name[1:-1]} {self.value})"
         out = [repr(self.name), self.first, self.second, self.third]
         out = [str(i) for i in out if i]
         return "(" + " ".join(out) + ")"
@@ -188,10 +188,7 @@ class Parser:
 
     def parse(self, program):
         generator = self.lexer.tokenize(program)
-        try:
-            self.next = generator.__next__  # PY3
-        except AttributeError:
-            self.next = generator.next
+        self.next = generator.__next__
         self.token = self.next()
         return self.expression()
 
@@ -354,11 +351,7 @@ class Evaluator:
         :type program: str or rope.base.oi.type_hinting.evaluate.SymbolBase
         :rtype: rope.base.pyobjects.PyDefinedObject | rope.base.pyobjects.PyObject or None
         """
-        ast = (
-            self.compile(program)
-            if isinstance(program, pycompat.string_types)
-            else program
-        )
+        ast = self.compile(program) if isinstance(program, str) else program
         return ast.evaluate(pyobject)
 
 
