@@ -1,13 +1,16 @@
+import contextlib
+import json
 import os
-import shutil
 import sys
 import warnings
+from contextlib import ExitStack
+from typing import Optional
 
-import rope.base.fscommands
+import rope.base.fscommands  # Use full qualification for clarity.
 import rope.base.resourceobserver as resourceobserver
-import rope.base.utils.pycompat as pycompat
-from rope.base import exceptions, taskhandle, prefs, history, pycore, utils
+from rope.base import exceptions, history, pycore, taskhandle, utils
 from rope.base.exceptions import ModuleNotFoundError
+from rope.base.prefs import Prefs, get_config
 from rope.base.resources import File, Folder, _ResourceMatcher
 
 try:
@@ -17,10 +20,12 @@ except ImportError:
 
 
 class _Project:
+    prefs: Prefs
+
     def __init__(self, fscommands):
         self.observers = []
         self.fscommands = fscommands
-        self.prefs = prefs.Prefs()
+        self.prefs = Prefs()
         self.data_files = _DataFiles(self)
         self._custom_source_folders = []
 
@@ -61,11 +66,9 @@ class _Project:
     def get_python_path_folders(self):
         result = []
         for src in self.prefs.get("python_path", []) + sys.path:
-            try:
+            with contextlib.suppress(exceptions.ResourceNotFoundError):
                 src_folder = get_no_project().get_resource(src)
                 result.append(src_folder)
-            except exceptions.ResourceNotFoundError:
-                pass
         return result
 
     # INFO: It was decided not to cache source folders, since:
@@ -103,7 +106,7 @@ class _Project:
         if observer in self.observers:
             self.observers.remove(observer)
 
-    def do(self, changes, task_handle=taskhandle.NullTaskHandle()):
+    def do(self, changes, task_handle=taskhandle.DEFAULT_TASK_HANDLE):
         """Apply the changes in a `ChangeSet`
 
         Most of the time you call this function for committing the
@@ -134,7 +137,7 @@ class _Project:
             raise ModuleNotFoundError("Module %s not found" % name)
         return self.pycore.resource_to_pyobject(module)
 
-    def find_module(self, modname, folder=None):
+    def find_module(self, modname, folder=None) -> Optional[File]:
         """Returns a resource corresponding to the given module
 
         returns None if it can not be found
@@ -204,6 +207,7 @@ class Project(_Project):
         """
         if projectroot != "/":
             projectroot = _realpath(projectroot).rstrip("/\\")
+        assert isinstance(projectroot, str)
         self._address = projectroot
         self._ropefolder_name = ropefolder
         if not os.path.exists(self._address):
@@ -215,12 +219,18 @@ class Project(_Project):
         super().__init__(fscommands)
         self.ignored = _ResourceMatcher()
         self.file_list = _FileListCacher(self)
-        self.prefs.add_callback("ignored_resources", self.ignored.set_patterns)
-        if ropefolder is not None:
-            self.prefs["ignored_resources"] = [ropefolder]
         self._init_prefs(prefs)
+        if ropefolder is not None:
+            self.prefs.add("ignored_resources", ropefolder)
         self._init_source_folders()
 
+    def __repr__(self):
+        return '<{}.{} "{}">'.format(
+            self.__class__.__module__,
+            self.__class__.__name__,
+            self.address,
+        )
+
     @utils.deprecated("Delete once deprecated functions are gone")
     def _init_source_folders(self):
         for path in self.prefs.get("source_folders", []):
@@ -242,12 +252,8 @@ class Project(_Project):
         return os.path.join(self._address, *name.split("/"))
 
     def _init_ropefolder(self):
-        if self.ropefolder is not None:
-            if not self.ropefolder.exists():
-                self._create_recursively(self.ropefolder)
-            if not self.ropefolder.has_child("config.py"):
-                config = self.ropefolder.create_file("config.py")
-                config.write(self._default_config())
+        if self.ropefolder is not None and not self.ropefolder.exists():
+            self._create_recursively(self.ropefolder)
 
     def _create_recursively(self, folder):
         if folder.parent != self.root and not folder.parent.exists():
@@ -255,39 +261,20 @@ class Project(_Project):
         folder.create()
 
     def _init_prefs(self, prefs):
-        run_globals = {}
-        if self.ropefolder is not None:
-            config = self.get_file(self.ropefolder.path + "/config.py")
-            run_globals.update(
-                {
-                    "__name__": "__main__",
-                    "__builtins__": __builtins__,
-                    "__file__": config.real_path,
-                }
-            )
-            if config.exists():
-                config = self.ropefolder.get_child("config.py")
-                pycompat.execfile(config.real_path, run_globals)
-            else:
-                exec(self._default_config(), run_globals)
-            if "set_prefs" in run_globals:
-                run_globals["set_prefs"](self.prefs)
+        config = get_config(self.root, self.ropefolder).parse()
+        self.prefs = config
+        self.prefs.add_callback("ignored_resources", self.ignored.set_patterns)
+        self.ignored.set_patterns(self.prefs.ignored_resources)
         for key, value in prefs.items():
-            self.prefs[key] = value
+            self.prefs.set(key, value)
         self._init_other_parts()
         self._init_ropefolder()
-        if "project_opened" in run_globals:
-            run_globals["project_opened"](self)
-
-    def _default_config(self):
-        import rope.base.default_config
-        import inspect
-
-        return inspect.getsource(rope.base.default_config)
+        if config.project_opened:
+            config.project_opened(self)
 
     def _init_other_parts(self):
         # Forcing the creation of `self.pycore` to register observers
-        self.pycore
+        self.pycore  # pylint: disable=pointless-statement
 
     def is_ignored(self, resource):
         return self.ignored.does_match(resource)
@@ -386,40 +373,31 @@ class _DataFiles:
         self.project = project
         self.hooks = []
 
-    def read_data(self, name, compress=False, import_=False):
+    def read_data(self, name):
         if self.project.ropefolder is None:
             return None
-        compress = compress and self._can_compress()
-        opener = self._get_opener(compress)
-        file = self._get_file(name, compress)
-        if not compress and import_:
-            self._import_old_files(name)
+        file = self._get_file(name)
         if file.exists():
-            input = opener(file.real_path, "rb")
-            try:
+            with open(file.real_path, "rb") as input_file:
                 result = []
                 try:
                     while True:
-                        result.append(pickle.load(input))
+                        result.append(pickle.load(input_file))
                 except EOFError:
                     pass
                 if len(result) == 1:
                     return result[0]
                 if len(result) > 1:
                     return result
-            finally:
-                input.close()
 
-    def write_data(self, name, data, compress=False):
+    def write_data(self, name, data):
         if self.project.ropefolder is not None:
-            compress = compress and self._can_compress()
-            file = self._get_file(name, compress)
-            opener = self._get_opener(compress)
-            output = opener(file.real_path, "wb")
-            try:
-                pickle.dump(data, output, 2)
-            finally:
-                output.close()
+            file = self._get_file(name)
+            with ExitStack() as cm:
+                output_file = cm.enter_context(open(file.real_path, "wb"))
+                output_file2 = cm.enter_context(open(file.real_path + ".json", "w"))
+                pickle.dump(data, output_file, 2)
+                json.dump(data, output_file2, default=lambda o: o.__getstate__())
 
     def add_write_hook(self, hook):
         self.hooks.append(hook)
@@ -428,34 +406,8 @@ class _DataFiles:
         for hook in self.hooks:
             hook()
 
-    def _can_compress(self):
-        try:
-            import gzip  # noqa
-
-            return True
-        except ImportError:
-            return False
-
-    def _import_old_files(self, name):
-        old = self._get_file(name + ".pickle", False)
-        new = self._get_file(name, False)
-        if old.exists() and not new.exists():
-            shutil.move(old.real_path, new.real_path)
-
-    def _get_opener(self, compress):
-        if compress:
-            try:
-                import gzip
-
-                return gzip.open
-            except ImportError:
-                pass
-        return open
-
-    def _get_file(self, name, compress):
+    def _get_file(self, name):
         path = self.project.ropefolder.path + "/" + name
-        if compress:
-            path += ".gz"
         return self.project.get_file(path)
 
 
