@@ -1,21 +1,15 @@
 import bisect
+import contextlib
 import difflib
-import sys
 import warnings
 
 import rope.base.libutils
-import rope.base.resourceobserver
-import rope.base.resources
 import rope.base.oi.doa
 import rope.base.oi.objectinfo
 import rope.base.oi.soa
-from rope.base import builtins
-from rope.base import exceptions
-from rope.base import stdmods
-from rope.base import taskhandle
-from rope.base import utils
-from rope.base.exceptions import ModuleNotFoundError
-from rope.base.pyobjectsdef import PyModule, PyPackage
+import rope.base.resourceobserver
+import rope.base.resources
+from rope.base import builtins, exceptions, pyobjectsdef, stdmods, taskhandle, utils
 
 
 class PyCore:
@@ -100,7 +94,7 @@ class PyCore:
         ``ignore_syntax_errors`` project config.
 
         """
-        return PyModule(self, code, resource, force_errors=force_errors)
+        return pyobjectsdef.PyModule(self, code, resource, force_errors=force_errors)
 
     @utils.deprecated("Use `libutils.get_string_scope` instead")
     def get_string_scope(self, code, resource=None):
@@ -146,13 +140,10 @@ class PyCore:
         return self.project.get_python_files()
 
     def _is_package(self, folder):
-        if (
+        return (
             folder.has_child("__init__.py")
             and not folder.get_child("__init__.py").is_folder()
-        ):
-            return True
-        else:
-            return False
+        )
 
     def _find_source_folders(self, folder):
         for resource in folder.get_folders():
@@ -217,7 +208,8 @@ class PyCore:
             self, pymodule, should_analyze, search_subscopes, followed_calls
         )
 
-    def get_classes(self, task_handle=taskhandle.NullTaskHandle()):
+    def get_classes(self, task_handle=taskhandle.DEFAULT_TASK_HANDLE):
+
         warnings.warn(
             "`PyCore.get_classes()` is deprecated", DeprecationWarning, stacklevel=2
         )
@@ -256,9 +248,17 @@ class _ModuleCache:
         if resource in self.module_map:
             return self.module_map[resource]
         if resource.is_folder():
-            result = PyPackage(self.pycore, resource, force_errors=force_errors)
+            result = pyobjectsdef.PyPackage(
+                self.pycore,
+                resource,
+                force_errors=force_errors,
+            )
         else:
-            result = PyModule(self.pycore, resource=resource, force_errors=force_errors)
+            result = pyobjectsdef.PyModule(
+                self.pycore,
+                resource=resource,
+                force_errors=force_errors,
+            )
             if result.has_errors:
                 return result
         self.module_map[resource] = result
@@ -290,7 +290,7 @@ class _ExtensionCache:
 def perform_soa_on_changed_scopes(project, resource, old_contents):
     pycore = project.pycore
     if resource.exists() and pycore.is_python_file(resource):
-        try:
+        with contextlib.suppress(exceptions.ModuleSyntaxError):
             new_contents = resource.read()
             # detecting changes in new_contents relative to old_contents
             detector = _TextChangeDetector(new_contents, old_contents)
@@ -306,8 +306,6 @@ def perform_soa_on_changed_scopes(project, resource, old_contents):
                 return detector.consume_changes(start, end)
 
             pycore.analyze_module(resource, should_analyze, search_subscopes)
-        except exceptions.ModuleSyntaxError:
-            pass
 
 
 class _TextChangeDetector:
@@ -335,9 +333,7 @@ class _TextChangeDetector:
         The end points are inclusive and indices start from 1.
         """
         left, right = self._get_changed(start, end)
-        if left < right:
-            return True
-        return False
+        return left < right
 
     def consume_changes(self, start, end):
         """Clear the changed status of lines from start till end"""
