@@ -1,4 +1,5 @@
-from rope.base.fscommands import _decode_data
+from typing import Optional
+
 from rope.base import ast, exceptions, utils
 
 
@@ -171,6 +172,19 @@ class PyDefinedObject:
         self.attributes = self.get_module()._get_concluded_data()
         self.defineds = None
 
+    def __repr__(self):
+        return '<{}.{} "{}" at {}>'.format(
+            self.__class__.__module__,
+            self.__class__.__name__,
+            self.absolute_name,
+            hex(id(self)),
+        )
+
+    @property
+    def absolute_name(self):
+        obj_name = self.get_name()
+        return self.get_module().get_name() + ("::" + obj_name if obj_name else "")
+
     visitor_class = None
 
     @utils.prevent_recursion(lambda: {})
@@ -211,13 +225,14 @@ class PyDefinedObject:
             current_object = current_object.parent
         return current_object
 
-    def get_doc(self):
+    def get_doc(self) -> Optional[str]:
         if len(self.get_ast().body) > 0:
             expr = self.get_ast().body[0]
             if isinstance(expr, ast.Expr) and isinstance(expr.value, ast.Str):
                 docstring = expr.value.s
-                coding = self.get_module().coding
-                return _decode_data(docstring, coding)
+                assert isinstance(docstring, str)
+                return docstring
+        return None
 
     def _get_defined_objects(self):
         if self.defineds is None:
@@ -228,8 +243,8 @@ class PyDefinedObject:
         if self.visitor_class is None:
             return {}
         new_visitor = self.visitor_class(self.pycore, self)
-        for child in ast.get_child_nodes(self.ast_node):
-            ast.walk(child, new_visitor)
+        for child in ast.iter_child_nodes(self.ast_node):
+            new_visitor.visit(child)
         self.defineds = new_visitor.defineds
         return new_visitor.names
 
@@ -244,15 +259,18 @@ class PyDefinedObject:
 
 
 class PyFunction(PyDefinedObject, AbstractFunction):
-    """Only a placeholder"""
+    pass
 
 
 class PyComprehension(PyDefinedObject, PyObject):
-    """Only a placeholder"""
+    pass
+
+    def get_name(self):
+        return "<comprehension>"
 
 
 class PyClass(PyDefinedObject, AbstractClass):
-    """Only a placeholder"""
+    pass
 
 
 class _ConcludedData:
@@ -281,6 +299,10 @@ class _PyModule(PyDefinedObject, AbstractModule):
         AbstractModule.__init__(self)
         PyDefinedObject.__init__(self, pycore, ast_node, None)
 
+    @property
+    def absolute_name(self) -> str:
+        return self.get_name()
+
     def _get_concluded_data(self):
         new_data = _ConcludedData()
         self.concluded_data.append(new_data)
@@ -295,11 +317,11 @@ class _PyModule(PyDefinedObject, AbstractModule):
 
 
 class PyModule(_PyModule):
-    """Only a placeholder"""
+    pass
 
 
 class PyPackage(_PyModule):
-    """Only a placeholder"""
+    pass
 
 
 class IsBeingInferredError(exceptions.RopeError):
