@@ -1,4 +1,3 @@
-from rope.base.pynames import DefinedName
 import rope.base.builtins
 import rope.base.codeanalyze
 import rope.base.evaluate
@@ -6,21 +5,15 @@ import rope.base.libutils
 import rope.base.oi.soi
 import rope.base.pyscopes
 from rope.base import (
-    pynamesdef as pynames,
-    exceptions,
-    ast,
-    astutils,
-    pyobjects,
-    fscommands,
     arguments,
+    ast,
+    exceptions,
+    fscommands,
+    nameanalyze,
+    pynamesdef,
+    pyobjects,
     utils,
 )
-from rope.base.utils import pycompat
-
-try:
-    unicode
-except NameError:
-    unicode = str
 
 
 class PyFunction(pyobjects.PyFunction):
@@ -28,10 +21,10 @@ class PyFunction(pyobjects.PyFunction):
         rope.base.pyobjects.AbstractFunction.__init__(self)
         rope.base.pyobjects.PyDefinedObject.__init__(self, pycore, ast_node, parent)
         self.arguments = self.ast_node.args
-        self.parameter_pyobjects = pynames._Inferred(
+        self.parameter_pyobjects = pynamesdef._Inferred(
             self._infer_parameters, self.get_module()._get_concluded_data()
         )
-        self.returned = pynames._Inferred(self._infer_returned)
+        self.returned = pynamesdef._Inferred(self._infer_returned)
         self.parameter_pynames = None
 
     def _create_structural_attributes(self):
@@ -68,7 +61,7 @@ class PyFunction(pyobjects.PyFunction):
             result = {}
             for index, name in enumerate(self.get_param_names()):
                 # TODO: handle tuple parameters
-                result[name] = pynames.ParameterName(self, index)
+                result[name] = pynamesdef.ParameterName(self, index)
             self.parameter_pynames = result
         return self.parameter_pynames
 
@@ -84,16 +77,12 @@ class PyFunction(pyobjects.PyFunction):
 
     def get_param_names(self, special_args=True):
         # TODO: handle tuple parameters
-        result = [
-            pycompat.get_ast_arg_arg(node)
-            for node in self.arguments.args
-            if isinstance(node, pycompat.ast_arg_type)
-        ]
+        result = [node.arg for node in self.arguments.args if isinstance(node, ast.arg)]
         if special_args:
             if self.arguments.vararg:
-                result.append(pycompat.get_ast_arg_arg(self.arguments.vararg))
+                result.append(self.arguments.vararg.arg)
             if self.arguments.kwarg:
-                result.append(pycompat.get_ast_arg_arg(self.arguments.kwarg))
+                result.append(self.arguments.kwarg.arg)
         return result
 
     def get_kind(self):
@@ -117,7 +106,7 @@ class PyFunction(pyobjects.PyFunction):
     @property
     def decorators(self):
         try:
-            return getattr(self.ast_node, "decorator_list")
+            return self.ast_node.decorator_list
         except AttributeError:
             return getattr(self.ast_node, "decorators", None)
 
@@ -204,7 +193,7 @@ class PyModule(pyobjects.PyModule):
                 source_bytes = resource.read_bytes()
                 source_code, _ = fscommands.file_data_to_unicode(source_bytes)
             else:
-                if isinstance(source_code, unicode):
+                if isinstance(source_code, str):
                     source_bytes = fscommands.unicode_to_file_data(source_code)
                 else:
                     source_bytes = source_code
@@ -262,7 +251,7 @@ class PyPackage(pyobjects.PyPackage):
         if self.resource is None:
             return result
         for name, resource in self._get_child_resources().items():
-            result[name] = pynames.ImportedModule(self, resource=resource)
+            result[name] = pynamesdef.ImportedModule(self, resource=resource)
         return result
 
     def _create_concluded_attributes(self):
@@ -302,7 +291,7 @@ class PyPackage(pyobjects.PyPackage):
         return rope.base.libutils.modname(self.resource) if self.resource else ""
 
 
-class _AnnAssignVisitor:
+class _AnnAssignVisitor(ast.RopeNodeVisitor):
     def __init__(self, scope_visitor):
         self.scope_visitor = scope_visitor
         self.assigned_ast = None
@@ -312,23 +301,23 @@ class _AnnAssignVisitor:
         self.assigned_ast = node.value
         self.type_hint = node.annotation
 
-        ast.walk(node.target, self)
+        self.visit(node.target)
 
     def _assigned(self, name, assignment=None):
         self.scope_visitor._assigned(name, assignment)
 
     def _Name(self, node):
-        assignment = pynames.AssignmentValue(
+        assignment = pynamesdef.AssignmentValue(
             self.assigned_ast, assign_type=True, type_hint=self.type_hint
         )
         self._assigned(node.id, assignment)
 
     def _Tuple(self, node):
-        names = astutils.get_name_levels(node)
+        names = nameanalyze.get_name_levels(node)
         for name, levels in names:
             assignment = None
             if self.assigned_ast is not None:
-                assignment = pynames.AssignmentValue(self.assigned_ast, levels)
+                assignment = pynamesdef.AssignmentValue(self.assigned_ast, levels)
             self._assigned(name, assignment)
 
     def _Annotation(self, node):
@@ -344,7 +333,7 @@ class _AnnAssignVisitor:
         pass
 
 
-class _ExpressionVisitor:
+class _ExpressionVisitor(ast.RopeNodeVisitor):
     def __init__(self, scope_visitor):
         self.scope_visitor = scope_visitor
 
@@ -367,11 +356,11 @@ class _ExpressionVisitor:
         self._GeneratorExp(node)
 
     def _NamedExpr(self, node):
-        ast.walk(node.target, _AssignVisitor(self))
-        ast.walk(node.value, self)
+        _AssignVisitor(self).visit(node.target)
+        self.visit(node.value)
 
 
-class _AssignVisitor:
+class _AssignVisitor(ast.RopeNodeVisitor):
     def __init__(self, scope_visitor):
         self.scope_visitor = scope_visitor
         self.assigned_ast = None
@@ -379,8 +368,8 @@ class _AssignVisitor:
     def _Assign(self, node):
         self.assigned_ast = node.value
         for child_node in node.targets:
-            ast.walk(child_node, self)
-        ast.walk(node.value, _ExpressionVisitor(self.scope_visitor))
+            self.visit(child_node)
+        _ExpressionVisitor(self.scope_visitor).visit(node.value)
 
     def _assigned(self, name, assignment=None):
         self.scope_visitor._assigned(name, assignment)
@@ -388,15 +377,15 @@ class _AssignVisitor:
     def _Name(self, node):
         assignment = None
         if self.assigned_ast is not None:
-            assignment = pynames.AssignmentValue(self.assigned_ast)
+            assignment = pynamesdef.AssignmentValue(self.assigned_ast)
         self._assigned(node.id, assignment)
 
     def _Tuple(self, node):
-        names = astutils.get_name_levels(node)
+        names = nameanalyze.get_name_levels(node)
         for name, levels in names:
             assignment = None
             if self.assigned_ast is not None:
-                assignment = pynames.AssignmentValue(self.assigned_ast, levels)
+                assignment = pynamesdef.AssignmentValue(self.assigned_ast, levels)
             self._assigned(name, assignment)
 
     def _Attribute(self, node):
@@ -425,7 +414,7 @@ class _ScopeVisitor(_ExpressionVisitor):
 
     def _ClassDef(self, node):
         pyclass = PyClass(self.pycore, node, self.owner_object)
-        self.names[node.name] = pynames.DefinedName(pyclass)
+        self.names[node.name] = pynamesdef.DefinedName(pyclass)
         self.defineds.append(pyclass)
 
     def _FunctionDef(self, node):
@@ -434,7 +423,7 @@ class _ScopeVisitor(_ExpressionVisitor):
             if isinstance(decorator, ast.Name) and decorator.id == "property":
                 if isinstance(self, _ClassVisitor):
                     type_ = rope.base.builtins.Property(pyfunction)
-                    arg = pynames.UnboundName(
+                    arg = pynamesdef.UnboundName(
                         rope.base.pyobjects.PyObject(self.owner_object)
                     )
 
@@ -445,32 +434,30 @@ class _ScopeVisitor(_ExpressionVisitor):
 
                     lineno = utils.guess_def_lineno(self.get_module(), node)
 
-                    self.names[node.name] = pynames.EvaluatedName(
+                    self.names[node.name] = pynamesdef.EvaluatedName(
                         _eval, module=self.get_module(), lineno=lineno
                     )
                     break
         else:
-            self.names[node.name] = pynames.DefinedName(pyfunction)
+            self.names[node.name] = pynamesdef.DefinedName(pyfunction)
         self.defineds.append(pyfunction)
 
     def _AsyncFunctionDef(self, node):
         return self._FunctionDef(node)
 
     def _Assign(self, node):
-        ast.walk(node, _AssignVisitor(self))
+        _AssignVisitor(self).visit(node)
 
     def _AnnAssign(self, node):
-        ast.walk(node, _AnnAssignVisitor(self))
+        _AnnAssignVisitor(self).visit(node)
 
     def _AugAssign(self, node):
         pass
 
     def _For(self, node):
-        names = self._update_evaluated(
-            node.target, node.iter, ".__iter__().next()"  # noqa
-        )
+        self._update_evaluated(node.target, node.iter, ".__iter__().next()")
         for child in node.body + node.orelse:
-            ast.walk(child, self)
+            self.visit(child)
 
     def _AsyncFor(self, node):
         return self._For(node)
@@ -478,8 +465,8 @@ class _ScopeVisitor(_ExpressionVisitor):
     def _assigned(self, name, assignment):
         pyname = self.names.get(name, None)
         if pyname is None:
-            pyname = pynames.AssignedName(module=self.get_module())
-        if isinstance(pyname, pynames.AssignedName):
+            pyname = pynamesdef.AssignedName(module=self.get_module())
+        if isinstance(pyname, pynamesdef.AssignedName):
             if assignment is not None:
                 pyname.assignments.append(assignment)
             self.names[name] = pyname
@@ -489,25 +476,25 @@ class _ScopeVisitor(_ExpressionVisitor):
     ):
         result = {}
         if isinstance(targets, str):
-            assignment = pynames.AssignmentValue(assigned, [], evaluation, eval_type)
+            assignment = pynamesdef.AssignmentValue(assigned, [], evaluation, eval_type)
             self._assigned(targets, assignment)
         else:
-            names = astutils.get_name_levels(targets)
+            names = nameanalyze.get_name_levels(targets)
             for name, levels in names:
-                assignment = pynames.AssignmentValue(
+                assignment = pynamesdef.AssignmentValue(
                     assigned, levels, evaluation, eval_type
                 )
                 self._assigned(name, assignment)
         return result
 
     def _With(self, node):
-        for item in pycompat.get_ast_with_items(node):
+        for item in node.items:
             if item.optional_vars:
                 self._update_evaluated(
                     item.optional_vars, item.context_expr, ".__enter__()"
                 )
         for child in node.body:
-            ast.walk(child, self)
+            self.visit(child)
 
     def _AsyncWith(self, node):
         return self._With(node)
@@ -521,7 +508,7 @@ class _ScopeVisitor(_ExpressionVisitor):
             self._update_evaluated(node.name, type_node, eval_type=True)
 
         for child in node.body:
-            ast.walk(child, self)
+            self.visit(child)
 
     def _ExceptHandler(self, node):
         self._excepthandler(node)
@@ -532,11 +519,11 @@ class _ScopeVisitor(_ExpressionVisitor):
             alias = import_pair.asname
             first_package = module_name.split(".")[0]
             if alias is not None:
-                imported = pynames.ImportedModule(self.get_module(), module_name)
+                imported = pynamesdef.ImportedModule(self.get_module(), module_name)
                 if not self._is_ignored_import(imported):
                     self.names[alias] = imported
             else:
-                imported = pynames.ImportedModule(self.get_module(), first_package)
+                imported = pynamesdef.ImportedModule(self.get_module(), first_package)
                 if not self._is_ignored_import(imported):
                     self.names[first_package] = imported
 
@@ -544,7 +531,11 @@ class _ScopeVisitor(_ExpressionVisitor):
         level = 0
         if node.level:
             level = node.level
-        imported_module = pynames.ImportedModule(self.get_module(), node.module, level)
+        imported_module = pynamesdef.ImportedModule(
+            self.get_module(),
+            node.module or "",
+            level,
+        )
         if self._is_ignored_import(imported_module):
             return
         if len(node.names) == 1 and node.names[0].name == "*":
@@ -556,7 +547,7 @@ class _ScopeVisitor(_ExpressionVisitor):
                 alias = imported_name.asname
                 if alias is not None:
                     imported = alias
-                self.names[imported] = pynames.ImportedName(
+                self.names[imported] = pynamesdef.ImportedName(
                     imported_module, imported_name.name
                 )
 
@@ -574,21 +565,21 @@ class _ScopeVisitor(_ExpressionVisitor):
                 try:
                     pyname = module[name]
                 except exceptions.AttributeNotFoundError:
-                    pyname = pynames.AssignedName(node.lineno)
+                    pyname = pynamesdef.AssignedName(node.lineno)
             self.names[name] = pyname
 
 
 class _ComprehensionVisitor(_ScopeVisitor):
     def _comprehension(self, node):
-        ast.walk(node.target, self)
-        ast.walk(node.iter, self)
+        self.visit(node.target)
+        self.visit(node.iter)
 
     def _Name(self, node):
         if isinstance(node.ctx, ast.Store):
             self.names[node.id] = self._get_pyobject(node)
 
     def _get_pyobject(self, node):
-        return pynames.AssignedName(lineno=node.lineno, module=self.get_module())
+        return pynamesdef.AssignedName(lineno=node.lineno, module=self.get_module())
 
 
 class _GlobalVisitor(_ScopeVisitor):
@@ -605,11 +596,11 @@ class _ClassVisitor(_ScopeVisitor):
         if len(node.args.args) > 0:
             first = node.args.args[0]
             new_visitor = None
-            if isinstance(first, pycompat.ast_arg_type):
-                new_visitor = _ClassInitVisitor(self, pycompat.get_ast_arg_arg(first))
+            if isinstance(first, ast.arg):
+                new_visitor = _ClassInitVisitor(self, first.arg)
             if new_visitor is not None:
-                for child in ast.get_child_nodes(node):
-                    ast.walk(child, new_visitor)
+                for child in ast.iter_child_nodes(node):
+                    new_visitor.visit(child)
 
 
 class _FunctionVisitor(_ScopeVisitor):
@@ -638,21 +629,21 @@ class _ClassInitVisitor(_AssignVisitor):
             return
         if isinstance(node.value, ast.Name) and node.value.id == self.self_name:
             if node.attr not in self.scope_visitor.names:
-                self.scope_visitor.names[node.attr] = pynames.AssignedName(
+                self.scope_visitor.names[node.attr] = pynamesdef.AssignedName(
                     lineno=node.lineno, module=self.scope_visitor.get_module()
                 )
             if self.assigned_ast is not None:
                 pyname = self.scope_visitor.names[node.attr]
-                if isinstance(pyname, pynames.AssignedName):
+                if isinstance(pyname, pynamesdef.AssignedName):
                     pyname.assignments.append(
-                        pynames.AssignmentValue(self.assigned_ast)
+                        pynamesdef.AssignmentValue(self.assigned_ast)
                     )
 
     def _Tuple(self, node):
         if not isinstance(node.ctx, ast.Store):
             return
-        for child in ast.get_child_nodes(node):
-            ast.walk(child, self)
+        for child in ast.iter_child_nodes(node):
+            self.visit(child)
 
     def _Name(self, node):
         pass
@@ -679,5 +670,5 @@ class StarImport:
         imported = self.imported_module.get_object()
         for name in imported:
             if not name.startswith("_"):
-                result[name] = pynames.ImportedName(self.imported_module, name)
+                result[name] = pynamesdef.ImportedName(self.imported_module, name)
         return result
