@@ -1,17 +1,54 @@
-import rope.base.evaluate
-from rope.base import libutils
-from rope.base import change, pyobjects, exceptions, pynames, worder, codeanalyze
-from rope.refactor import sourceutils, importutils, functionutils, suites
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
+
+from rope.base import (
+    change,
+    codeanalyze,
+    evaluate,
+    exceptions,
+    libutils,
+    pynames,
+    pyobjects,
+    worder,
+)
+from rope.refactor import functionutils, importutils, sourceutils, suites
+
+if TYPE_CHECKING:
+    from typing import Literal, Optional
+
+    from rope.base.project import Project
+    from rope.base.resources import Resource
+
+    GenerateKind = Literal[
+        "variable",
+        "function",
+        "class",
+        "module",
+        "package",
+    ]
 
 
-def create_generate(kind, project, resource, offset, goal_resource=None):
+def create_generate(
+    kind: GenerateKind,
+    project: Project,
+    resource: Resource,
+    offset: int,
+    goal_resource: Optional[Resource] = None,
+):
+
     """A factory for creating `Generate` objects
 
-    `kind` can be 'variable', 'function', 'class', 'module' or
-    'package'.
-
+    Used in https://github.com/python-rope/ropemode but not in Rope itself.
     """
-    generate = eval("Generate" + kind.title())
+    d = {
+        "class": GenerateClass,
+        "function": GenerateFunction,
+        "module": GenerateModule,
+        "package": GeneratePackage,
+        "variable": GenerateVariable,
+    }
+    generate = d[kind]
     return generate(project, resource, offset, goal_resource=goal_resource)
 
 
@@ -39,7 +76,7 @@ def create_package(project, name, sourcefolder=None):
     return made_packages
 
 
-class _Generate(object):
+class _Generate:
     def __init__(self, project, resource, offset, goal_resource=None):
         self.project = project
         self.resource = resource
@@ -62,9 +99,7 @@ class _Generate(object):
             )
 
     def get_changes(self):
-        changes = change.ChangeSet(
-            "Generate %s <%s>" % (self._get_element_kind(), self.name)
-        )
+        changes = change.ChangeSet(f"Generate {self._get_element_kind()} <{self.name}>")
         indents = self.info.get_scope_indents()
         blanks = self.info.get_blank_lines()
         base_definition = sourceutils.fix_indentation(self._get_element(), indents)
@@ -109,7 +144,7 @@ class GenerateFunction(_Generate):
         ):
             args.append("self")
         args.extend(self.info.get_passed_args())
-        definition = "%sdef %s(%s):\n    pass\n" % (
+        definition = "{}def {}({}):\n    pass\n".format(
             decorator,
             self.name,
             ", ".join(args),
@@ -140,7 +175,7 @@ class GenerateModule(_Generate):
     def get_changes(self):
         package = self.info.get_package()
         changes = change.ChangeSet("Generate Module <%s>" % self.name)
-        new_resource = self.project.get_file("%s/%s.py" % (package.path, self.name))
+        new_resource = self.project.get_file(f"{package.path}/{self.name}.py")
         if new_resource.exists():
             raise exceptions.RefactoringError(
                 "Module <%s> already exists" % new_resource.path
@@ -160,7 +195,7 @@ class GeneratePackage(_Generate):
     def get_changes(self):
         package = self.info.get_package()
         changes = change.ChangeSet("Generate Package <%s>" % self.name)
-        new_resource = self.project.get_folder("%s/%s" % (package.path, self.name))
+        new_resource = self.project.get_folder(f"{package.path}/{self.name}")
         if new_resource.exists():
             raise exceptions.RefactoringError(
                 "Package <%s> already exists" % new_resource.path
@@ -198,14 +233,14 @@ def _add_relative_import_to_module(project, resource, imported, name):
     return change.ChangeContents(resource, module_imports.get_changed_source())
 
 
-class _GenerationInfo(object):
+class _GenerationInfo:
     def __init__(self, pycore, resource, offset, goal_resource=None):
         self.pycore = pycore
         self.resource = resource
         self.offset = offset
         self.goal_resource = goal_resource
         self.source_pymodule = self.pycore.project.get_pymodule(resource)
-        finder = rope.base.evaluate.ScopeNameFinder(self.source_pymodule)
+        finder = evaluate.ScopeNameFinder(self.source_pymodule)
         self.primary, self.pyname = finder.get_primary_and_pyname_at(offset)
         self._init_fields()
 
