@@ -1,12 +1,12 @@
 import re
 from contextlib import contextmanager
+from itertools import chain
 
 from rope.base import ast, codeanalyze
-from rope.base.change import ChangeSet, ChangeContents
+from rope.base.change import ChangeContents, ChangeSet
 from rope.base.exceptions import RefactoringError
-from rope.base.utils import pycompat
 from rope.base.utils.datastructures import OrderedSet
-from rope.refactor import sourceutils, similarfinder, patchedast, suites, usefunction
+from rope.refactor import patchedast, similarfinder, sourceutils, suites, usefunction
 
 
 # Extract refactoring has lots of special cases.  I tried to split it
@@ -33,7 +33,7 @@ from rope.refactor import sourceutils, similarfinder, patchedast, suites, usefun
 #
 # There are a few more helper functions and classes used by above
 # classes.
-class _ExtractRefactoring(object):
+class _ExtractRefactoring:
 
     kind_prefixes = {}
 
@@ -80,7 +80,7 @@ class _ExtractRefactoring(object):
         )
         info.kind = self._get_kind(kind)
         new_contents = _ExtractPerformer(info).extract()
-        changes = ChangeSet("Extract %s <%s>" % (info.kind, extracted_name))
+        changes = ChangeSet(f"Extract {info.kind} <{extracted_name}>")
         changes.add_change(ChangeContents(self.resource, new_contents))
         return changes
 
@@ -98,7 +98,7 @@ class _ExtractRefactoring(object):
 
     @classmethod
     def _get_kind(cls, kind):
-        raise NotImplementedError("You have to sublass {}".format(cls))
+        raise NotImplementedError(f"You have to subclass {cls}")
 
 
 class ExtractMethod(_ExtractRefactoring):
@@ -115,7 +115,7 @@ class ExtractVariable(_ExtractRefactoring):
     def __init__(self, *args, **kwds):
         kwds = dict(kwds)
         kwds["variable"] = True
-        super(ExtractVariable, self).__init__(*args, **kwds)
+        super().__init__(*args, **kwds)
 
     kind = "variable"
 
@@ -123,7 +123,7 @@ class ExtractVariable(_ExtractRefactoring):
         return cls.kind
 
 
-class _ExtractInfo(object):
+class _ExtractInfo:
     """Holds information about the extract to be performed"""
 
     def __init__(
@@ -222,13 +222,13 @@ class _ExtractInfo(object):
     def extracted(self):
         return self.source[self.region[0] : self.region[1]]
 
-    _cached_parsed_extraced = None
+    _cached_parsed_extracted = None
 
     @property
     def _parsed_extracted(self):
-        if self._cached_parsed_extraced is None:
-            self._cached_parsed_extraced = _parse_text(self.extracted)
-        return self._cached_parsed_extraced
+        if self._cached_parsed_extracted is None:
+            self._cached_parsed_extracted = _parse_text(self.extracted)
+        return self._cached_parsed_extracted
 
     _returned = None
 
@@ -250,8 +250,22 @@ class _ExtractInfo(object):
             )
         return self._returning_named_expr
 
+    _returning_generator = None
 
-class _ExtractCollector(object):
+    @property
+    def returning_generator_exp(self):
+        """Does the extracted piece contains a generator expression"""
+        if self._returning_generator is None:
+            self._returning_generator = (
+                isinstance(self._parsed_extracted, ast.Module)
+                and isinstance(self._parsed_extracted.body[0], ast.Expr)
+                and isinstance(self._parsed_extracted.body[0].value, ast.GeneratorExp)
+            )
+
+        return self._returning_generator
+
+
+class _ExtractCollector:
     """Collects information needed for performing the extract"""
 
     def __init__(self, info):
@@ -264,7 +278,7 @@ class _ExtractCollector(object):
         self.definition_location = None
 
 
-class _ExtractPerformer(object):
+class _ExtractPerformer:
     def __init__(self, info):
         self.info = info
         _ExceptionalConditionChecker()(self.info)
@@ -375,7 +389,7 @@ class _ExtractPerformer(object):
         collector.checks = parts.get_checks()
 
 
-class _DefinitionLocationFinder(object):
+class _DefinitionLocationFinder:
     def __init__(self, info, matched_lines):
         self.info = info
         self.matched_lines = matched_lines
@@ -419,7 +433,7 @@ class _DefinitionLocationFinder(object):
         return self.info.scope.get_end() + 1
 
 
-class _ExceptionalConditionChecker(object):
+class _ExceptionalConditionChecker:
     def __call__(self, info):
         self.base_conditions(info)
         if info.one_line:
@@ -500,7 +514,7 @@ class _ExceptionalConditionChecker(object):
         return next.isalnum() or next == "_"
 
 
-class _ExtractMethodParts(object):
+class _ExtractMethodParts(ast.RopeNodeVisitor):
     def __init__(self, info):
         self.info = info
         self.info_collector = self._create_info_collector()
@@ -575,7 +589,7 @@ class _ExtractMethodParts(object):
         )
         body = self.info.source[self.info.scope_region[0] : self.info.scope_region[1]]
         node = _parse_text(body)
-        ast.walk(node, info_collector)
+        info_collector.visit(node)
         return info_collector
 
     def _get_function_definition(self):
@@ -709,7 +723,7 @@ class _ExtractMethodParts(object):
 
     def _get_unindented_function_body(self, returns):
         if self.info.one_line:
-            return self._get_one_line_function_body()
+            return self._get_single_expression_function_body()
         return self._get_multiline_function_body(returns)
 
     def _get_multiline_function_body(self, returns):
@@ -719,11 +733,9 @@ class _ExtractMethodParts(object):
             unindented_body += "\nreturn %s" % self._get_comma_form(returns)
         return unindented_body
 
-    def _get_one_line_function_body(self):
-        if self.info.returning_named_expr:
-            body = "return " + "(" + _join_lines(self.info.extracted) + ")"
-        else:
-            body = "return " + _join_lines(self.info.extracted)
+    def _get_single_expression_function_body(self):
+        extracted = _get_single_expression_body(self.info.extracted, info=self.info)
+        body = "return " + extracted
         return self._insert_globals(body)
 
     def _insert_globals(self, unindented_body):
@@ -743,17 +755,17 @@ class _ExtractMethodParts(object):
     def _get_globals_in_body(unindented_body):
         node = _parse_text(unindented_body)
         visitor = _GlobalFinder()
-        ast.walk(node, visitor)
+        visitor.visit(node)
         return visitor.globals_
 
 
-class _ExtractVariableParts(object):
+class _ExtractVariableParts:
     def __init__(self, info):
         self.info = info
 
     def get_definition(self):
-        result = self.info.new_name + " = " + _join_lines(self.info.extracted) + "\n"
-        return result
+        extracted = _get_single_expression_body(self.info.extracted, info=self.info)
+        return self.info.new_name + " = " + extracted + "\n"
 
     def get_body_pattern(self):
         return "(%s)" % self.info.extracted.strip()
@@ -765,7 +777,7 @@ class _ExtractVariableParts(object):
         return {}
 
 
-class _FunctionInformationCollector(object):
+class _FunctionInformationCollector(ast.RopeNodeVisitor):
     def __init__(self, start, end, is_global):
         self.start = start
         self.end = end
@@ -810,12 +822,12 @@ class _FunctionInformationCollector(object):
             for name in _get_argnames(node.args):
                 self._written_variable(name, node.lineno)
             for child in node.body:
-                ast.walk(child, self)
+                self.visit(child)
         else:
             self._written_variable(node.name, node.lineno)
             visitor = _VariableReadsAndWritesFinder()
             for child in node.body:
-                ast.walk(child, visitor)
+                visitor.visit(child)
             for name in visitor.read - visitor.written:
                 self._read_variable(name, node.lineno)
 
@@ -831,15 +843,24 @@ class _FunctionInformationCollector(object):
         if not isinstance(node.ctx, ast.Store):
             self._read_variable(node.id, node.lineno)
 
+    def _MatchAs(self, node):
+        self._written_variable(node.name, node.lineno)
+        if node.pattern:
+            self.visit(node.pattern)
+
     def _Assign(self, node):
-        ast.walk(node.value, self)
+        self.visit(node.value)
         for child in node.targets:
-            ast.walk(child, self)
+            self.visit(child)
 
     def _AugAssign(self, node):
-        ast.walk(node.value, self)
-        self._read_variable(node.target.id, node.target.lineno)
-        self._written_variable(node.target.id, node.target.lineno)
+        self.visit(node.value)
+        if isinstance(node.target, ast.Name):
+            target_id = node.target.id
+            self._read_variable(target_id, node.target.lineno)
+            self._written_variable(target_id, node.target.lineno)
+        else:
+            self.visit(node.target)
 
     def _ClassDef(self, node):
         self._written_variable(node.name, node.lineno)
@@ -861,14 +882,28 @@ class _FunctionInformationCollector(object):
         written = OrderedSet(self.written)
         maybe_written = OrderedSet(self.maybe_written)
 
-        for child in ast.get_child_nodes(node):
-            ast.walk(child, self)
+        for child in ast.iter_child_nodes(node):
+            self.visit(child)
 
-        comp_names = set([name.target.id for name in node.generators])
+        comp_names = list(
+            chain.from_iterable(
+                self._flatten_nested_tuple_of_names(generator.target)
+                for generator in node.generators
+            )
+        )
         self.read = self.read - comp_names | read
         self.written = self.written - comp_names | written
         self.maybe_written = self.maybe_written - comp_names | maybe_written
 
+    def _flatten_nested_tuple_of_names(self, node):
+        if isinstance(node, ast.Tuple):
+            for elt in node.elts:
+                yield self._flatten_nested_tuple_of_names(elt)
+        elif isinstance(node, ast.Name):
+            yield node.id
+        else:
+            assert False, f"Unexpected node type in list comprehension target: {node!r}"
+
     def _If(self, node):
         self._handle_conditional_node(node)
 
@@ -879,18 +914,18 @@ class _FunctionInformationCollector(object):
     def _For(self, node):
         with self._handle_loop_context(node), self._handle_conditional_context(node):
             # iter has to be checked before the target variables
-            ast.walk(node.iter, self)
-            ast.walk(node.target, self)
+            self.visit(node.iter)
+            self.visit(node.target)
 
             for child in node.body:
-                ast.walk(child, self)
+                self.visit(child)
             for child in node.orelse:
-                ast.walk(child, self)
+                self.visit(child)
 
     def _handle_conditional_node(self, node):
         with self._handle_conditional_context(node):
-            for child in ast.get_child_nodes(node):
-                ast.walk(child, self)
+            for child in ast.iter_child_nodes(node):
+                self.visit(child)
 
     @contextmanager
     def _handle_conditional_context(self, node):
@@ -912,19 +947,18 @@ class _FunctionInformationCollector(object):
 
 
 def _get_argnames(arguments):
-    result = [
-        pycompat.get_ast_arg_arg(node)
-        for node in arguments.args
-        if isinstance(node, pycompat.ast_arg_type)
-    ]
+    result = []
+    result.extend(node.arg for node in getattr(arguments, "posonlyargs", []))
+    result.extend(node.arg for node in arguments.args)
     if arguments.vararg:
-        result.append(pycompat.get_ast_arg_arg(arguments.vararg))
+        result.append(arguments.vararg.arg)
     if arguments.kwarg:
-        result.append(pycompat.get_ast_arg_arg(arguments.kwarg))
+        result.append(arguments.kwarg.arg)
+    result.extend(node.arg for node in arguments.kwonlyargs)
     return result
 
 
-class _VariableReadsAndWritesFinder(object):
+class _VariableReadsAndWritesFinder(ast.RopeNodeVisitor):
     def __init__(self):
         self.written = set()
         self.read = set()
@@ -938,8 +972,8 @@ class _VariableReadsAndWritesFinder(object):
     def _FunctionDef(self, node):
         self.written.add(node.name)
         visitor = _VariableReadsAndWritesFinder()
-        for child in ast.get_child_nodes(node):
-            ast.walk(child, visitor)
+        for child in ast.iter_child_nodes(node):
+            visitor.visit(child)
         self.read.update(visitor.read - visitor.written)
 
     def _Class(self, node):
@@ -951,7 +985,7 @@ class _VariableReadsAndWritesFinder(object):
             return set(), set()
         node = _parse_text(code)
         visitor = _VariableReadsAndWritesFinder()
-        ast.walk(node, visitor)
+        visitor.visit(node)
         return visitor.read, visitor.written
 
     @staticmethod
@@ -960,18 +994,18 @@ class _VariableReadsAndWritesFinder(object):
             return set(), set()
         node = _parse_text(code)
         visitor = _VariableReadsAndWritesFinder()
-        ast.walk(node, visitor)
+        visitor.visit(node)
         return visitor.read
 
 
-class _BaseErrorFinder(object):
+class _BaseErrorFinder(ast.RopeNodeVisitor):
     @classmethod
     def has_errors(cls, code):
         if code.strip() == "":
             return False
         node = _parse_text(code)
         visitor = cls()
-        ast.walk(node, visitor)
+        visitor.visit(node)
         return visitor.error
 
 
@@ -989,14 +1023,14 @@ class _UnmatchedBreakOrContinueFinder(_BaseErrorFinder):
     def loop_encountered(self, node):
         self.loop_count += 1
         for child in node.body:
-            ast.walk(child, self)
+            self.visit(child)
         self.loop_count -= 1
         if node.orelse:
             if isinstance(node.orelse, (list, tuple)):
                 for node_ in node.orelse:
-                    ast.walk(node_, self)
+                    self.visit(node_)
             else:
-                ast.walk(node.orelse, self)
+                self.visit(node.orelse)
 
     def _Break(self, node):
         self.check_loop()
@@ -1032,7 +1066,7 @@ class _AsyncStatementFinder(_BaseErrorFinder):
         pass
 
 
-class _GlobalFinder(object):
+class _GlobalFinder(ast.RopeNodeVisitor):
     def __init__(self):
         self.globals_ = OrderedSet()
 
@@ -1069,3 +1103,21 @@ def _join_lines(code):
         else:
             lines.append(line.strip())
     return " ".join(lines)
+
+
+def _get_single_expression_body(extracted, info):
+    extracted = sourceutils.fix_indentation(extracted, 0)
+    already_parenthesized = (
+        extracted.lstrip()[0] in "({[" and extracted.rstrip()[-1] in ")}]"
+    )
+    large_multiline = extracted.count("\n") >= 2 and already_parenthesized
+    if not large_multiline:
+        extracted = _join_lines(extracted)
+    multiline_expression = "\n" in extracted
+    if (
+        info.returning_named_expr
+        or info.returning_generator_exp
+        or (multiline_expression and not large_multiline)
+    ):
+        extracted = "(" + extracted + ")"
+    return extracted
