@@ -4,22 +4,30 @@
 based on inputs.
 
 """
+from __future__ import annotations
+
+import typing
+from typing import Union
+
 from rope.base import (
-    pyobjects,
     codeanalyze,
-    exceptions,
-    pynames,
-    taskhandle,
     evaluate,
-    worder,
+    exceptions,
     libutils,
+    pynames,
+    pyobjects,
+    taskhandle,
+    worder,
 )
-from rope.base.change import ChangeSet, ChangeContents, MoveResource
-from rope.refactor import importutils, rename, occurrences, sourceutils, functionutils
+from rope.base.change import ChangeContents, ChangeSet, MoveResource
+from rope.refactor import functionutils, importutils, occurrences, rename, sourceutils
 from rope.refactor.importutils.module_imports import (
     get_first_decorator_or_function_start_line,
 )
 
+if typing.TYPE_CHECKING:
+    from rope.base import project, resources
+
 
 def create_move(project, resource, offset=None):
     """A factory for creating Move objects
@@ -49,7 +57,7 @@ def create_move(project, resource, offset=None):
         ):
             return MoveGlobal(project, resource, offset)
     raise exceptions.RefactoringError(
-        "Move only works on global classes/functions/variables, modules and " "methods."
+        "Move only works on global classes/functions/variables, modules and methods."
     )
 
 
@@ -69,14 +77,14 @@ class MoveMethod:
         self.method_name = worder.get_name_at(resource, offset)
         self.pyfunction = pyname.get_object()
         if self.pyfunction.get_kind() != "method":
-            raise exceptions.RefactoringError("Only normal methods" " can be moved.")
+            raise exceptions.RefactoringError("Only normal methods can be moved.")
 
     def get_changes(
         self,
         dest_attr,
         new_name=None,
         resources=None,
-        task_handle=taskhandle.NullTaskHandle(),
+        task_handle=taskhandle.DEFAULT_TASK_HANDLE,  # FIXME: this is unused
     ):
         """Return the changes needed for this refactoring
 
@@ -231,6 +239,8 @@ class MoveMethod:
 class MoveGlobal:
     """For moving global function and classes"""
 
+    project: project.Project
+
     def __init__(self, project, resource, offset):
         self.project = project
         this_pymodule = self.project.get_pymodule(resource)
@@ -299,8 +309,23 @@ class MoveGlobal:
         return isinstance(pyname, pynames.AssignedName)
 
     def get_changes(
-        self, dest, resources=None, task_handle=taskhandle.NullTaskHandle()
+        self,
+        dest: Union[None, str, resources.Resource],
+        resources=None,
+        task_handle=taskhandle.DEFAULT_TASK_HANDLE,
     ):
+        """Return the changes needed for this refactoring
+
+        Parameters:
+
+        - `dest`: the Resource or dotted moddule name of the move destination
+        - `resources` can be a list of `rope.base.resources.File` to
+          apply this refactoring on.  If `None`, the restructuring
+          will be applied to all python files.
+
+        """
+        if isinstance(dest, str):
+            dest = self.project.find_module(dest)
         if resources is None:
             resources = self.project.get_python_files()
         if dest is None or not dest.exists():
@@ -487,7 +512,7 @@ class MoveModule:
         self.import_tools = self.tools.import_tools
 
     def get_changes(
-        self, dest, resources=None, task_handle=taskhandle.NullTaskHandle()
+        self, dest, resources=None, task_handle=taskhandle.DEFAULT_TASK_HANDLE
     ):
         if resources is None:
             resources = self.project.get_python_files()
@@ -706,9 +731,9 @@ class _MoveTools:
 
     def occurs_in_module(self, pymodule=None, resource=None, imports=True):
         finder = self._create_finder(imports)
-        for occurrence in finder.find_occurrences(pymodule=pymodule, resource=resource):
-            return True
-        return False
+        occurrences = finder.find_occurrences(pymodule=pymodule, resource=resource)
+        sentinel = object()
+        return next(occurrences, sentinel) is not sentinel
 
     def _create_finder(self, imports):
         return occurrences.create_finder(
@@ -759,14 +784,9 @@ def moving_code_with_imports(project, resource, source):
 
     origin = project.get_pymodule(resource)
 
-    imports = []
-    for stmt in import_tools.module_imports(origin).imports:
-        imports.append(stmt.import_info)
+    imports = [stmt.import_info for stmt in import_tools.module_imports(origin).imports]
 
-    back_names = []
-    for name in origin:
-        if name not in pymodule:
-            back_names.append(name)
+    back_names = [name for name in origin if name not in pymodule]
     imports.append(import_tools.get_from_import(resource, back_names))
 
     source = _add_imports_to_module(import_tools, pymodule, imports)
