@@ -5,13 +5,6 @@ import warnings
 from itertools import chain
 
 from rope.base import ast, codeanalyze, exceptions
-from rope.base.utils import pycompat
-
-
-try:
-    basestring
-except NameError:
-    basestring = (str, bytes)
 
 COMMA_IN_WITH_PATTERN = re.compile(r"\(.*?\)|(,)")
 
@@ -41,7 +34,7 @@ def patch_ast(node, source, sorted_children=False):
     if hasattr(node, "region"):
         return node
     walker = _PatchingASTWalker(source, children=sorted_children)
-    ast.call_for_nodes(node, walker)
+    walker(node)
     return node
 
 
@@ -78,10 +71,6 @@ class _PatchingASTWalker:
 
     Number = object()
     String = object()
-    semicolon_or_as_in_except = object()
-    exec_open_paren_or_space = object()
-    exec_close_paren_or_space = object()
-    exec_in_or_comma = object()
     with_or_comma_context_manager = object()
     empty_tuple = object()
 
@@ -96,7 +85,7 @@ class _PatchingASTWalker:
         )
         node.region = (self.source.offset, self.source.offset)
         if self.children:
-            node.sorted_children = ast.get_children(node)
+            node.sorted_children = [child for field, child in ast.iter_fields(node)]
 
     def _handle(self, node, base_children, eat_parens=False, eat_spaces=False):
         if hasattr(node, "region"):
@@ -121,7 +110,7 @@ class _PatchingASTWalker:
                 continue
             offset = self.source.offset
             if isinstance(child, ast.AST):
-                ast.call_for_nodes(child, self)
+                self(child)
                 token_start = child.region[0]
             else:
                 if child is self.String:
@@ -132,35 +121,17 @@ class _PatchingASTWalker:
                     region = self.source.consume_number()
                 elif child == self.empty_tuple:
                     region = self.source.consume_empty_tuple()
-                elif child == "!=":
-                    # INFO: This has been added to handle deprecated ``<>``
-                    region = self.source.consume_not_equal()
-                elif child == self.semicolon_or_as_in_except:
-                    # INFO: This has been added to handle deprecated
-                    # semicolon in except
-                    region = self.source.consume_except_as_or_semicolon()
-                elif child == self.exec_open_paren_or_space:
-                    # These three cases handle the differences between
-                    # the deprecated exec statement and the exec
-                    # function.
-                    region = self.source.consume_exec_open_paren_or_space()
-                elif child == self.exec_in_or_comma:
-                    region = self.source.consume_exec_in_or_comma()
-                elif child == self.exec_close_paren_or_space:
-                    region = self.source.consume_exec_close_paren_or_space()
                 elif child == self.with_or_comma_context_manager:
                     region = self.source.consume_with_or_comma_context_manager()
+                elif isinstance(node, (ast.JoinedStr, ast.FormattedValue)):
+                    region = self.source.consume_joined_string(child)
                 else:
-                    if hasattr(ast, "JoinedStr") and isinstance(
-                        node, (ast.JoinedStr, ast.FormattedValue)
-                    ):
-                        region = self.source.consume_joined_string(child)
-                    else:
-                        region = self.source.consume(child)
+                    region = self.source.consume(child)
                 child = self.source[region[0] : region[1]]
                 token_start = region[0]
             if not first_token:
-                formats.append(self.source[offset:token_start])
+                if not isinstance(node, ast.JoinedStr):
+                    formats.append(self.source[offset:token_start])
                 if self.children:
                     children.append(self.source[offset:token_start])
             else:
@@ -191,9 +162,8 @@ class _PatchingASTWalker:
         new_end = None
         for i in range(closes):
             new_end = self.source.consume(")")[1]
-        if new_end is not None:
-            if self.children:
-                children.append(self.source[old_end:new_end])
+        if new_end is not None and self.children:
+            children.append(self.source[old_end:new_end])
         new_start = start
         for i in range(opens):
             new_start = self.source.rfind_token("(", 0, new_start)
@@ -222,7 +192,7 @@ class _PatchingASTWalker:
         start = 0
         opens = 0
         for child in children:
-            if not isinstance(child, basestring):
+            if not isinstance(child, (str, bytes)):
                 continue
             if child == "" or child[0] in "'\"":
                 continue
@@ -314,15 +284,11 @@ class _PatchingASTWalker:
         self._handle(node, children)
 
     def _Assign(self, node):
-        children = self._child_nodes(node.targets, "=")
-        children.append("=")
-        children.append(node.value)
+        children = [*self._child_nodes(node.targets, "="), "=", node.value]
         self._handle(node, children)
 
     def _AugAssign(self, node):
-        children = [node.target]
-        children.extend(self._get_op(node.op))
-        children.extend(["=", node.value])
+        children = [node.target, *self._get_op(node.op), "=", node.value]
         self._handle(node, children)
 
     def _AnnAssign(self, node):
@@ -332,9 +298,6 @@ class _PatchingASTWalker:
             children.append(node.value)
         self._handle(node, children)
 
-    def _Repr(self, node):
-        self._handle(node, ["`", node.value, "`"])
-
     def _BinOp(self, node):
         children = [node.left] + self._get_op(node.op) + [node.right]
         self._handle(node, children)
@@ -352,43 +315,15 @@ class _PatchingASTWalker:
             return (node.lineno, node.col_offset)
 
         children = [node.func, "("]
-        unstarred_args = []
-        starred_and_keywords = list(node.keywords)
-        for i, arg in enumerate(node.args):
-            if hasattr(ast, "Starred") and isinstance(arg, ast.Starred):
-                starred_and_keywords.append(arg)
-            else:
-                unstarred_args.append(arg)
-        if getattr(node, "starargs", None):
-            starred_and_keywords.append(node.starargs)
-        starred_and_keywords.sort(key=_arg_sort_key)
-        children.extend(self._child_nodes(unstarred_args, ","))
-
-        # positional args come before keywords, *args comes after all
-        # positional args, and **kwargs comes last
-        if starred_and_keywords:
-            if len(children) > 2:
-                children.append(",")
-            for i, arg in enumerate(starred_and_keywords):
-                if arg == getattr(node, "starargs", None):
-                    children.append("*")
-                children.append(arg)
-                if i + 1 < len(starred_and_keywords):
-                    children.append(",")
-
-        if getattr(node, "kwargs", None):
-            if len(children) > 2:
-                children.append(",")
-            children.extend(["**", node.kwargs])
+        args = sorted([*node.args, *node.keywords], key=_arg_sort_key)
+        children.extend(self._child_nodes(args, ","))
         children.append(")")
         self._handle(node, children)
 
     def _ClassDef(self, node):
         children = []
-        if getattr(node, "decorator_list", None):
-            for decorator in node.decorator_list:
-                children.append("@")
-                children.append(decorator)
+        for decorator in node.decorator_list:
+            children.extend(("@", decorator))
         children.extend(["class", node.name])
         if node.bases:
             children.append("(")
@@ -410,7 +345,7 @@ class _PatchingASTWalker:
         self._handle(node, ["del"] + self._child_nodes(node.targets, ","))
 
     def _Constant(self, node):
-        if isinstance(node.value, basestring):
+        if isinstance(node.value, (str, bytes)):
             self._handle(node, [self.String])
             return
 
@@ -514,15 +449,6 @@ class _PatchingASTWalker:
         children = [node.target, ":=", node.value]
         self._handle(node, children)
 
-    def _Exec(self, node):
-        children = ["exec", self.exec_open_paren_or_space, node.body]
-        if node.globals:
-            children.extend([self.exec_in_or_comma, node.globals])
-        if node.locals:
-            children.extend([",", node.locals])
-        children.append(self.exec_close_paren_or_space)
-        self._handle(node, children)
-
     def _ExtSlice(self, node):
         children = []
         for index, dim in enumerate(node.dims):
@@ -532,10 +458,7 @@ class _PatchingASTWalker:
         self._handle(node, children)
 
     def _handle_for_loop_node(self, node, is_async):
-        if is_async:
-            children = ["async", "for"]
-        else:
-            children = ["for"]
+        children = ["async", "for"] if is_async else ["for"]
         children.extend([node.target, "in", node.iter, ":"])
         children.extend(node.body)
         if node.orelse:
@@ -552,34 +475,27 @@ class _PatchingASTWalker:
     def _ImportFrom(self, node):
         children = ["from"]
         if node.level:
-            children.append("." * node.level)
-        # see comment at rope.base.ast.walk
-        children.extend([node.module or "", "import"])
+            children.extend("." * node.level)
+        if node.module:
+            children.extend(node.module.split("."))
+        children.append("import")
         children.extend(self._child_nodes(node.names, ","))
         self._handle(node, children)
 
     def _alias(self, node):
-        children = [node.name]
+        children = node.name.split(".")
         if node.asname:
             children.extend(["as", node.asname])
         self._handle(node, children)
 
     def _handle_function_def_node(self, node, is_async):
         children = []
-        try:
-            decorators = getattr(node, "decorator_list")
-        except AttributeError:
-            decorators = getattr(node, "decorators", None)
-        if decorators:
-            for decorator in decorators:
-                children.append("@")
-                children.append(decorator)
-        if is_async:
-            children.extend(["async", "def"])
-        else:
-            children.extend(["def"])
-        children.extend([node.name, "(", node.args])
-        children.extend([")", ":"])
+        for decorator in node.decorator_list:
+            children.extend(("@", decorator))
+        children.extend(["async", "def"] if is_async else ["def"])
+        children.append(node.name)
+        children.extend(["(", node.args, ")"])
+        children.append(":")
         children.extend(node.body)
         self._handle(node, children)
 
@@ -600,11 +516,11 @@ class _PatchingASTWalker:
         if node.vararg is not None:
             if args:
                 children.append(",")
-            children.extend(["*", pycompat.get_ast_arg_arg(node.vararg)])
+            children.extend(["*", node.vararg.arg])
         if node.kwarg is not None:
             if args or node.vararg is not None:
                 children.append(",")
-            children.extend(["**", pycompat.get_ast_arg_arg(node.kwarg)])
+            children.extend(["**", node.kwarg.arg])
         self._handle(node, children)
 
     def _add_args_to_children(self, children, arg, default):
@@ -628,28 +544,21 @@ class _PatchingASTWalker:
         children.append(")")
 
     def _GeneratorExp(self, node):
-        children = [node.elt]
-        children.extend(node.generators)
+        children = [node.elt, *node.generators]
         self._handle(node, children, eat_parens=True)
 
     def _comprehension(self, node):
         children = ["for", node.target, "in", node.iter]
-        if node.ifs:
-            for if_ in node.ifs:
-                children.append("if")
-                children.append(if_)
+        for if_ in node.ifs:
+            children.extend(["if", if_])
         self._handle(node, children)
 
     def _Global(self, node):
-        children = self._child_nodes(node.names, ",")
-        children.insert(0, "global")
+        children = ["global", *self._child_nodes(node.names, ",")]
         self._handle(node, children)
 
     def _If(self, node):
-        if self._is_elif(node):
-            children = ["elif"]
-        else:
-            children = ["if"]
+        children = ["elif"] if self._is_elif(node) else ["if"]
         children.extend([node.test, ":"])
         children.extend(node.body)
         if node.orelse:
@@ -673,33 +582,29 @@ class _PatchingASTWalker:
         return self._handle(node, [node.body, "if", node.test, "else", node.orelse])
 
     def _Import(self, node):
-        children = ["import"]
-        children.extend(self._child_nodes(node.names, ","))
+        children = ["import", *self._child_nodes(node.names, ",")]
         self._handle(node, children)
 
     def _keyword(self, node):
-        children = []
         if node.arg is None:
-            children.append(node.value)
+            children = [node.value]
         else:
-            children.extend([node.arg, "=", node.value])
+            children = [node.arg, "=", node.value]
         self._handle(node, children)
 
     def _Lambda(self, node):
         self._handle(node, ["lambda", node.args, ":", node.body])
 
     def _List(self, node):
-        self._handle(node, ["["] + self._child_nodes(node.elts, ",") + ["]"])
+        self._handle(node, ["[", *self._child_nodes(node.elts, ","), "]"])
 
     def _ListComp(self, node):
-        children = ["[", node.elt]
-        children.extend(node.generators)
-        children.append("]")
+        children = ["[", node.elt, *node.generators, "]"]
         self._handle(node, children)
 
     def _Set(self, node):
         if node.elts:
-            self._handle(node, ["{"] + self._child_nodes(node.elts, ",") + ["}"])
+            self._handle(node, ["{", *self._child_nodes(node.elts, ","), "}"])
             return
         # Python doesn't have empty set literals
         warnings.warn(
@@ -708,16 +613,11 @@ class _PatchingASTWalker:
         self._handle(node, ["set(", ")"])
 
     def _SetComp(self, node):
-        children = ["{", node.elt]
-        children.extend(node.generators)
-        children.append("}")
+        children = ["{", node.elt, *node.generators, "}"]
         self._handle(node, children)
 
     def _DictComp(self, node):
-        children = ["{"]
-        children.extend([node.key, ":", node.value])
-        children.extend(node.generators)
-        children.append("}")
+        children = ["{", *[node.key, ":", node.value], *node.generators, "}"]
         self._handle(node, children)
 
     def _Module(self, node):
@@ -735,22 +635,12 @@ class _PatchingASTWalker:
     def _Pass(self, node):
         self._handle(node, ["pass"])
 
-    def _Print(self, node):
-        children = ["print"]
-        if node.dest:
-            children.extend([">>", node.dest])
-            if node.values:
-                children.append(",")
-        children.extend(self._child_nodes(node.values, ","))
-        if not node.nl:
-            children.append(",")
-        self._handle(node, children)
-
     def _Raise(self, node):
         children = ["raise"]
         if node.exc:
             children.append(node.exc)
         if node.cause:
+            children.append("from")
             children.append(node.cause)
         self._handle(node, children)
 
@@ -760,15 +650,6 @@ class _PatchingASTWalker:
             children.append(node.value)
         self._handle(node, children)
 
-    def _Sliceobj(self, node):
-        children = []
-        for index, slice in enumerate(node.nodes):
-            if index > 0:
-                children.append(":")
-            if slice:
-                children.append(slice)
-        self._handle(node, children)
-
     def _Index(self, node):
         self._handle(node, [node.value])
 
@@ -827,12 +708,11 @@ class _PatchingASTWalker:
         self._excepthandler(node)
 
     def _excepthandler(self, node):
-        # self._handle(node, [self.semicolon_or_as_in_except])
         children = ["except"]
         if node.type:
             children.append(node.type)
         if node.name:
-            children.append(self.semicolon_or_as_in_except)
+            children.append("as")
             children.append(node.name)
         children.append(":")
         children.extend(node.body)
@@ -879,7 +759,7 @@ class _PatchingASTWalker:
 
         if is_async:
             children.extend(["async"])
-        for item in pycompat.get_ast_with_items(node):
+        for item in node.items:
             children.extend([self.with_or_comma_context_manager, item.context_expr])
             if item.optional_vars:
                 children.extend(["as", item.optional_vars])
@@ -919,16 +799,28 @@ class _PatchingASTWalker:
         self._handle(node, children)
 
     def _MatchClass(self, node):
-        children = []
-        children.extend([node.cls, "("])
-        children.extend(self._child_nodes(node.patterns, ","))
-        children.extend(self._flatten_keywords(zip(node.kwd_attrs, node.kwd_patterns)))
-        children.append(")")
+        children = [
+            node.cls,
+            "(",
+            *self._child_nodes(node.patterns, ","),
+            *self._flatten_keywords(zip(node.kwd_attrs, node.kwd_patterns)),
+            ")",
+        ]
         self._handle(node, children)
 
     def _MatchValue(self, node):
         self._handle(node, [node.value])
 
+    def _MatchMapping(self, node):
+        children = []
+        children.append("{")
+        for index, (key, value) in enumerate(zip(node.keys, node.patterns)):
+            children.extend([key, ":", value])
+            if index < len(node.keys) - 1:
+                children.append(",")
+        children.append("}")
+        self._handle(node, children)
+
 
 class _Source:
     def __init__(self, source):
@@ -943,9 +835,9 @@ class _Source:
                     break
                 else:
                     self._skip_comment()
-        except (ValueError, TypeError) as e:
+        except (ValueError, TypeError):
             raise MismatchedTokenError(
-                "Token <{}> at {} cannot be matched".format(token, self._get_location())
+                f"Token <{token}> at {self._get_location()} cannot be matched"
             )
         self.offset = new_offset + len(token)
         return (new_offset, self.offset)
@@ -959,8 +851,14 @@ class _Source:
         if _Source._string_pattern is None:
             string_pattern = codeanalyze.get_string_pattern()
             formatted_string_pattern = codeanalyze.get_formatted_string_pattern()
-            original = r"(?:{})|(?:{})".format(string_pattern, formatted_string_pattern)
-            pattern = r"({})((\s|\\\n|#[^\n]*\n)*({}))*".format(original, original)
+            original = r"(?:{})|(?:{})".format(
+                string_pattern,
+                formatted_string_pattern,
+            )
+            pattern = r"({})((\s|\\\n|#[^\n]*\n)*({}))*".format(
+                original,
+                original,
+            )
             _Source._string_pattern = re.compile(pattern)
         repattern = _Source._string_pattern
         return self._consume_pattern(repattern, end)
@@ -974,28 +872,6 @@ class _Source:
     def consume_empty_tuple(self):
         return self._consume_pattern(re.compile(r"\(\s*\)"))
 
-    def consume_not_equal(self):
-        if _Source._not_equals_pattern is None:
-            _Source._not_equals_pattern = re.compile(r"<>|!=")
-        repattern = _Source._not_equals_pattern
-        return self._consume_pattern(repattern)
-
-    def consume_except_as_or_semicolon(self):
-        repattern = re.compile(r"as|,")
-        return self._consume_pattern(repattern)
-
-    def consume_exec_open_paren_or_space(self):
-        repattern = re.compile(r"\(|")
-        return self._consume_pattern(repattern)
-
-    def consume_exec_in_or_comma(self):
-        repattern = re.compile(r"in|,")
-        return self._consume_pattern(repattern)
-
-    def consume_exec_close_paren_or_space(self):
-        repattern = re.compile(r"\)|")
-        return self._consume_pattern(repattern)
-
     def consume_with_or_comma_context_manager(self):
         repattern = re.compile(r"with|,")
         return self._consume_pattern(repattern)
@@ -1063,9 +939,8 @@ class _Source:
 
     def _get_number_pattern(self):
         # HACK: It is merely an approaximation and does the job
-        integer = r"\-?(0x[\da-fA-F]+|\d+)[lL]?"
+        integer = r"\-?(0[xo][\da-fA-F]+|\d+)"
         return r"(%s(\.\d*)?|(\.\d+))([eE][-+]?\d+)?[jJ]?" % integer
 
     _string_pattern = None
     _number_pattern = None
-    _not_equals_pattern = None
