@@ -5,13 +5,6 @@ import pytest
 from ropetest import testutils
 
 
-@pytest.fixture
-def project():
-    project = testutils.sample_project()
-    yield project
-    testutils.remove_project(project)
-
-
 @pytest.fixture
 def mod1(project):
     mod1 = testutils.create_module(project, "mod1")
@@ -23,11 +16,6 @@ def mod1_path(mod1):
     yield pathlib.Path(mod1.real_path)
 
 
-@pytest.fixture
-def project_path(project):
-    yield pathlib.Path(project.address)
-
-
 @pytest.fixture
 def typing_path():
     import typing
