@@ -1,22 +1,13 @@
-try:
-    import unittest2 as unittest
-except ImportError:
-    import unittest
+import itertools
 import sys
+import unittest
 from textwrap import dedent
 
 from rope.base import ast
-from rope.base.utils import pycompat
 from rope.refactor import patchedast
 from ropetest import testutils
 
-try:
-    basestring
-except NameError:
-    basestring = (str, bytes)
-
 NameConstant = "Name" if sys.version_info <= (3, 8) else "NameConstant"
-Bytes = "Bytes" if (3, 0) <= sys.version_info <= (3, 8) else "Str"
 
 
 class PatchedASTTest(unittest.TestCase):
@@ -46,14 +37,27 @@ class PatchedASTTest(unittest.TestCase):
             "Expr",
         ])
 
+    def test_operator_support_completeness(self):
+        ast_ops = {
+            n.__name__
+            for n in itertools.chain(
+                ast.boolop.__subclasses__(),
+                ast.cmpop.__subclasses__(),
+                ast.operator.__subclasses__(),
+                ast.unaryop.__subclasses__(),
+            )
+        }
+        supported_ops = set(patchedast._PatchingASTWalker._operators)
+        assert ast_ops == supported_ops
+
     def test_bytes_string(self):
         source = '1 + b"("\n'
         ast_frag = patchedast.get_patched_ast(source, True)
         checker = _ResultChecker(self, ast_frag)
         str_fragment = 'b"("'
         start = source.index(str_fragment)
-        checker.check_region(Bytes, start, start + len(str_fragment))
-        checker.check_children(Bytes, [str_fragment])
+        checker.check_region("Bytes", start, start + len(str_fragment))
+        checker.check_children("Bytes", [str_fragment])
 
     def test_integer_literals_and_region(self):
         source = "a = 10\n"
@@ -87,28 +91,20 @@ class PatchedASTTest(unittest.TestCase):
         start = source.index("0x1")
         checker.check_region("Num", start, start + 3)
 
-    @testutils.only_for_versions_lower("3")
-    def test_long_literals_and_region(self):
-        source = "a = 0x1L\n"
-        ast_frag = patchedast.get_patched_ast(source, True)
-        checker = _ResultChecker(self, ast_frag)
-        start = source.index("0x1L")
-        checker.check_region("Num", start, start + 4)
-
     def test_octal_integer_literals_and_region(self):
-        source = "a = -0125e1\n"
+        source = "a = -0o1251\n"
         ast_frag = patchedast.get_patched_ast(source, True)
         checker = _ResultChecker(self, ast_frag)
-        start = source.index("-0125e1") + 1
+        start = source.index("-0o1251") + 1
         end = start + 6
         # Python 3 parses as UnaryOp(op=USub(), operand=Num(n=10))
         checker.check_region("Num", start, end)
+        checker.check_children("Num", ["0o1251"])
 
     def test_integer_literals_and_sorted_children(self):
         source = "a = 10\n"
         ast_frag = patchedast.get_patched_ast(source, True)
         checker = _ResultChecker(self, ast_frag)
-        # start = source.index('10')
         checker.check_children("Num", ["10"])
 
     def test_ellipsis(self):
@@ -130,7 +126,6 @@ class PatchedASTTest(unittest.TestCase):
         source = "a = 10\n"
         ast_frag = patchedast.get_patched_ast(source, True)
         checker = _ResultChecker(self, ast_frag)
-        start = source.index("a")  # noqa
         checker.check_region("Assign", 0, len(source) - 1)
         checker.check_children("Assign", ["Name", " ", "=", " ", "Num"])
 
@@ -139,7 +134,6 @@ class PatchedASTTest(unittest.TestCase):
         source = "a: List[int]\n"
         ast_frag = patchedast.get_patched_ast(source, True)
         checker = _ResultChecker(self, ast_frag)
-        start = source.index("a")  # noqa
         checker.check_region("AnnAssign", 0, len(source) - 1)
         checker.check_children("AnnAssign", ["Name", "", ":", " ", "Subscript"])
 
@@ -148,7 +142,6 @@ class PatchedASTTest(unittest.TestCase):
         source = "a: int = 10\n"
         ast_frag = patchedast.get_patched_ast(source, True)
         checker = _ResultChecker(self, ast_frag)
-        start = source.index("a")  # noqa
         checker.check_region("AnnAssign", 0, len(source) - 1)
         checker.check_children(
             "AnnAssign", ["Name", "", ":", " ", "Name", " ", "=", " ", "Num"]
@@ -227,6 +220,12 @@ class PatchedASTTest(unittest.TestCase):
         checker = _ResultChecker(self, ast_frag)
         checker.check_children("BinOp", ["Num", " ", "+", " ", "Str"])
 
+    def test_handling_fstrings(self):
+        source = '1 + f"("\n'
+        ast_frag = patchedast.get_patched_ast(source, True)
+        checker = _ResultChecker(self, ast_frag)
+        checker.check_children("BinOp", ["Num", " ", "+", " ", "JoinedStr"])
+
     def test_handling_implicit_string_concatenation(self):
         source = "a = '1''2'"
         ast_frag = patchedast.get_patched_ast(source, True)
@@ -280,7 +279,7 @@ class PatchedASTTest(unittest.TestCase):
         checker = _ResultChecker(self, ast_frag)
         checker.check_children(
             "JoinedStr",
-            ["rf'", "abc", "FormattedValue", '\' f"""xxx', "FormattedValue", " ", '"""',],
+            ["rf'", "abc", "FormattedValue", '\' f"""xxx', "FormattedValue", " ", '"""'],
         )
         checker.check_children("FormattedValue", ["{", "", "Name", "", "}"])
 
@@ -313,14 +312,6 @@ class PatchedASTTest(unittest.TestCase):
         checker.check_children("JoinedStr", ['f"', "abc", "FormattedValue", "", '"'])
         checker.check_children("FormattedValue", ["{", "", "BinOp", "", "}"])
 
-    @testutils.only_for_versions_lower("3")
-    def test_long_integer_literals(self):
-        source = "0x1L + a"
-        ast_frag = patchedast.get_patched_ast(source, True)
-        checker = _ResultChecker(self, ast_frag)
-        checker.check_children("BinOp", ["Num", " ", "+", " ", "Name"])
-        checker.check_children("Num", ["0x1L"])
-
     def test_complex_number_literals(self):
         source = "1.0e2j + a"
         ast_frag = patchedast.get_patched_ast(source, True)
@@ -378,18 +369,9 @@ class PatchedASTTest(unittest.TestCase):
         source = "a += 1\n"
         ast_frag = patchedast.get_patched_ast(source, True)
         checker = _ResultChecker(self, ast_frag)
-        start = source.index("a")  # noqa
         checker.check_region("AugAssign", 0, len(source) - 1)
         checker.check_children("AugAssign", ["Name", " ", "+", "", "=", " ", "Num"])
 
-    @testutils.only_for_versions_lower("3")
-    def test_back_quotenode(self):
-        source = "`1`\n"
-        ast_frag = patchedast.get_patched_ast(source, True)
-        checker = _ResultChecker(self, ast_frag)
-        checker.check_region("Repr", 0, len(source) - 1)
-        checker.check_children("Repr", ["`", "", "Num", "", "`"])
-
     def test_bitand(self):
         source = "1 & 2\n"
         ast_frag = patchedast.get_patched_ast(source, True)
@@ -429,7 +411,6 @@ class PatchedASTTest(unittest.TestCase):
             "Call", ["Name", "", "(", "", "Num", "", ",", " ", "*", "", "Name", "", ")"]
         )
 
-    @testutils.only_for("3.5")
     def test_call_func_and_star_argspython35(self):
         source = "f(1, *args)\n"
         ast_frag = patchedast.get_patched_ast(source, True)
@@ -445,7 +426,6 @@ class PatchedASTTest(unittest.TestCase):
         checker = _ResultChecker(self, ast_frag)
         checker.check_children("Call", ["Name", "", "(", "", "**", "", "Name", "", ")"])
 
-    @testutils.only_for("3.5")
     def test_call_func_and_only_dstar_args_python35(self):
         source = "f(**kwds)\n"
         ast_frag = patchedast.get_patched_ast(source, True)
@@ -462,7 +442,6 @@ class PatchedASTTest(unittest.TestCase):
             ["Name", "", "(", "", "*", "", "Name", "", ",", " ", "**", "", "Name", "", ")"],
         )
 
-    @testutils.only_for("3.5")
     def test_call_func_and_both_varargs_and_kwargs_python35(self):
         source = "f(*args, **kwds)\n"
         ast_frag = patchedast.get_patched_ast(source, True)
@@ -526,7 +505,6 @@ class PatchedASTTest(unittest.TestCase):
             ["@", "", "Name", "\n", "def", " ", "f", "", "(", "", "arguments", "", ")", "", ":", "\n    ", "Pass"],
         )
 
-    @testutils.only_for("2.6")
     def test_decorators_for_classes(self):
         source = dedent("""\
             @d
@@ -592,26 +570,11 @@ class PatchedASTTest(unittest.TestCase):
             "Function",
             ["def", " ", "f", "", "(", "", "arguments", "", ")", "", ":", "\n    ", "Expr", "\n    ", "Pass"],
         )
-        expected_child = pycompat.ast_arg_type.__name__
+        expected_child = ast.arg.__name__
         checker.check_children(
             "arguments", [expected_child, "", ",", " ", "**", "", "p2"]
         )
 
-    @testutils.only_for_versions_lower("3")
-    def test_function_node_and_tuple_parameters(self):
-        source = dedent("""\
-            def f(a, (b, c)):
-                pass
-        """)
-        ast_frag = patchedast.get_patched_ast(source, True)
-        checker = _ResultChecker(self, ast_frag)
-        checker.check_region("Function", 0, len(source) - 1)
-        checker.check_children(
-            "Function",
-            ["def", " ", "f", "", "(", "", "arguments", "", ")", "", ":", "\n    ", "Pass"],
-        )
-        checker.check_children("arguments", ["Name", "", ",", " ", "Tuple"])
-
     def test_dict_node(self):
         source = "{1: 2, 3: 4}\n"
         ast_frag = patchedast.get_patched_ast(source, True)
@@ -622,7 +585,6 @@ class PatchedASTTest(unittest.TestCase):
             ["{", "", "Num", "", ":", " ", "Num", "", ",", " ", "Num", "", ":", " ", "Num", "", "}"],
         )
 
-    @testutils.only_for("3.5")
     def test_dict_node_with_unpacking(self):
         source = "{**dict1, **dict2}\n"
         ast_frag = patchedast.get_patched_ast(source, True)
@@ -639,36 +601,6 @@ class PatchedASTTest(unittest.TestCase):
         checker.check_region("BinOp", 0, len(source) - 1)
         checker.check_children("BinOp", ["Num", " ", "/", " ", "Num"])
 
-    @testutils.only_for_versions_lower("3")
-    def test_simple_exec_node(self):
-        source = 'exec ""\n'
-        ast_frag = patchedast.get_patched_ast(source, True)
-        checker = _ResultChecker(self, ast_frag)
-        checker.check_region("Exec", 0, len(source) - 1)
-        checker.check_children("Exec", ["exec", "", "", " ", "Str", "", ""])
-
-    @testutils.only_for_versions_lower("3")
-    def test_exec_node(self):
-        source = 'exec "" in locals(), globals()\n'
-        ast_frag = patchedast.get_patched_ast(source, True)
-        checker = _ResultChecker(self, ast_frag)
-        checker.check_region("Exec", 0, len(source) - 1)
-        checker.check_children(
-            "Exec",
-            ["exec", "", "", " ", "Str", " ", "in", " ", "Call", "", ",", " ", "Call", "", ""],
-        )
-
-    @testutils.only_for_versions_lower("3")
-    def test_exec_node_with_parens(self):
-        source = 'exec("", locals(), globals())\n'
-        ast_frag = patchedast.get_patched_ast(source, True)
-        checker = _ResultChecker(self, ast_frag)
-        checker.check_region("Exec", 0, len(source) - 1)
-        checker.check_children(
-            "Exec",
-            ["exec", "", "(", "", "Str", "", ",", " ", "Call", "", ",", " ", "Call", "", ")"],
-        )
-
     def test_for_node(self):
         source = dedent("""\
             for i in range(1):
@@ -723,25 +655,43 @@ class PatchedASTTest(unittest.TestCase):
         )
         checker.check_children("alias", ["y"])
 
-    @testutils.only_for("2.5")
     def test_from_node(self):
         source = "from ..x import y as z\n"
         ast_frag = patchedast.get_patched_ast(source, True)
         checker = _ResultChecker(self, ast_frag)
         checker.check_region("ImportFrom", 0, len(source) - 1)
         checker.check_children(
-            "ImportFrom", ["from", " ", "..", "", "x", " ", "import", " ", "alias"]
+            "ImportFrom", ["from", " ", ".", "", ".", "", "x", " ", "import", " ", "alias"]
         )
         checker.check_children("alias", ["y", " ", "as", " ", "z"])
 
-    @testutils.only_for("2.5")
     def test_from_node_relative_import(self):
         source = "from . import y as z\n"
         ast_frag = patchedast.get_patched_ast(source, True)
         checker = _ResultChecker(self, ast_frag)
         checker.check_region("ImportFrom", 0, len(source) - 1)
         checker.check_children(
-            "ImportFrom", ["from", " ", ".", "", "", " ", "import", " ", "alias"]
+            "ImportFrom", ["from", " ", ".", " ", "import", " ", "alias"]
+        )
+        checker.check_children("alias", ["y", " ", "as", " ", "z"])
+
+    def test_from_node_whitespace_around_dots_1(self):
+        source = "from . . . import y as z\n"
+        ast_frag = patchedast.get_patched_ast(source, True)
+        checker = _ResultChecker(self, ast_frag)
+        checker.check_region("ImportFrom", 0, len(source) - 1)
+        checker.check_children(
+            "ImportFrom", ["from", " ", ".", " ", ".", " ", ".", " ", "import", " ", "alias"]
+        )
+        checker.check_children("alias", ["y", " ", "as", " ", "z"])
+
+    def test_from_node_whitespace_around_dots_2(self):
+        source = "from . a . b import y as z\n"
+        ast_frag = patchedast.get_patched_ast(source, True)
+        checker = _ResultChecker(self, ast_frag)
+        checker.check_region("ImportFrom", 0, len(source) - 1)
+        checker.check_children(
+            "ImportFrom", ["from", " ", ".", " ", "a", " . ", "b", " ", "import", " ", "alias"]
         )
         checker.check_children("alias", ["y", " ", "as", " ", "z"])
 
@@ -836,6 +786,15 @@ class PatchedASTTest(unittest.TestCase):
             "Import", ["import", " ", "alias", "", ",", " ", "alias"]
         )
 
+    def test_import_node_whitespace_around_dots(self):
+        source = "import a . b, b as c\n"
+        ast_frag = patchedast.get_patched_ast(source, True)
+        checker = _ResultChecker(self, ast_frag)
+        checker.check_region("Import", 0, len(source) - 1)
+        checker.check_children(
+            "Import", ["import", " ", "alias", "", ",", " ", "alias"]
+        )
+
     def test_lambda_node(self):
         source = "lambda a, b=1, *z: None\n"
         ast_frag = patchedast.get_patched_ast(source, True)
@@ -844,7 +803,7 @@ class PatchedASTTest(unittest.TestCase):
         checker.check_children(
             "Lambda", ["lambda", " ", "arguments", "", ":", " ", NameConstant]
         )
-        expected_child = pycompat.ast_arg_type.__name__
+        expected_child = ast.arg.__name__
         checker.check_children(
             "arguments",
             [expected_child, "", ",", " ", expected_child, "", "=", "", "Num", "", ",", " ", "*", "", "z"],
@@ -885,28 +844,14 @@ class PatchedASTTest(unittest.TestCase):
         )
 
     def test_set_node(self):
-        # make sure we are in a python version with set literals
         source = "{1, 2}\n"
-
-        try:
-            eval(source)
-        except SyntaxError:
-            return
-
         ast_frag = patchedast.get_patched_ast(source, True)
         checker = _ResultChecker(self, ast_frag)
         checker.check_region("Set", 0, len(source) - 1)
         checker.check_children("Set", ["{", "", "Num", "", ",", " ", "Num", "", "}"])
 
     def test_set_comp_node(self):
-        # make sure we are in a python version with set comprehensions
         source = "{i for i in range(1) if True}\n"
-
-        try:
-            eval(source)
-        except SyntaxError:
-            return
-
         ast_frag = patchedast.get_patched_ast(source, True)
         checker = _ResultChecker(self, ast_frag)
         checker.check_region("SetComp", 0, len(source) - 1)
@@ -919,14 +864,7 @@ class PatchedASTTest(unittest.TestCase):
         )
 
     def test_dict_comp_node(self):
-        # make sure we are in a python version with dict comprehensions
         source = "{i:i for i in range(1) if True}\n"
-
-        try:
-            eval(source)
-        except SyntaxError:
-            return
-
         ast_frag = patchedast.get_patched_ast(source, True)
         checker = _ResultChecker(self, ast_frag)
         checker.check_region("DictComp", 0, len(source) - 1)
@@ -975,36 +913,13 @@ class PatchedASTTest(unittest.TestCase):
         checker.check_children("Expr", ["BoolOp"])
         checker.check_children("BoolOp", ["UnaryOp", " ", "or", " ", NameConstant])
 
-    @testutils.only_for_versions_lower("3")
-    def test_print_node(self):
-        source = "print >>out, 1,\n"
-        ast_frag = patchedast.get_patched_ast(source, True)
-        checker = _ResultChecker(self, ast_frag)
-        checker.check_region("Print", 0, len(source) - 1)
-        checker.check_children(
-            "Print", ["print", " ", ">>", "", "Name", "", ",", " ", "Num", "", ","]
-        )
-
-    @testutils.only_for_versions_lower("3")
-    def test_printnl_node(self):
-        source = "print 1\n"
-        ast_frag = patchedast.get_patched_ast(source, True)
-        checker = _ResultChecker(self, ast_frag)
-        checker.check_region("Print", 0, len(source) - 1)
-        checker.check_children("Print", ["print", " ", "Num"])
-
-    @testutils.only_for_versions_lower("3")
-    def test_raise_node_for_python2(self):
-        source = "raise x, y, z\n"
+    def test_raise_node_bare(self):
+        source = "raise\n"
         ast_frag = patchedast.get_patched_ast(source, True)
         checker = _ResultChecker(self, ast_frag)
         checker.check_region("Raise", 0, len(source) - 1)
-        checker.check_children(
-            "Raise", ["raise", " ", "Name", "", ",", " ", "Name", "", ",", " ", "Name"]
-        )
+        checker.check_children("Raise", ["raise"])
 
-    # @#testutils.only_for('3')
-    @unittest.skipIf(sys.version < "3", "This is wrong")
     def test_raise_node_for_python3(self):
         source = "raise x(y)\n"
         ast_frag = patchedast.get_patched_ast(source, True)
@@ -1012,6 +927,13 @@ class PatchedASTTest(unittest.TestCase):
         checker.check_region("Raise", 0, len(source) - 1)
         checker.check_children("Raise", ["raise", " ", "Call"])
 
+    def test_raise_node_for_python3_with_cause(self):
+        source = "raise x(y) from e\n"
+        ast_frag = patchedast.get_patched_ast(source, True)
+        checker = _ResultChecker(self, ast_frag)
+        checker.check_region("Raise", 0, len(source) - 1)
+        checker.check_children("Raise", ["raise", " ", "Call", " ", "from", " ", "Name"])
+
     def test_return_node(self):
         source = dedent("""\
             def f():
@@ -1151,7 +1073,6 @@ class PatchedASTTest(unittest.TestCase):
             ["while", " ", NameConstant, "", ":", "\n    ", "Pass", "\n", "else", "", ":", "\n    ", "Pass"],
         )
 
-    @testutils.only_for("2.5")
     def test_with_node(self):
         source = dedent("""\
             from __future__ import with_statement
@@ -1165,7 +1086,6 @@ class PatchedASTTest(unittest.TestCase):
             ["with", " ", "Name", " ", "as", " ", "Name", "", ":", "\n    ", "Pass"],
         )
 
-    @testutils.only_for("3.5")
     def test_async_with_node(self):
         source = dedent("""\
             async def afunc():
@@ -1192,26 +1112,7 @@ class PatchedASTTest(unittest.TestCase):
         expected_children = ["try", "", ":", "\n    ", "Pass", "\n", "finally", "", ":", "\n    ", "Pass"]
         checker.check_children(node_to_test, expected_children)
 
-    @testutils.only_for_versions_lower("3")
     def test_try_except_node(self):
-        source = dedent("""\
-            try:
-                pass
-            except Exception, e:
-                pass
-        """)
-        ast_frag = patchedast.get_patched_ast(source, True)
-        checker = _ResultChecker(self, ast_frag)
-        checker.check_children(
-            "TryExcept",
-            ["try", "", ":", "\n    ", "Pass", "\n", ("excepthandler", "ExceptHandler")],
-        )
-        checker.check_children(
-            ("excepthandler", "ExceptHandler"),
-            ["except", " ", "Name", "", ",", " ", "Name", "", ":", "\n    ", "Pass"],
-        )
-
-    def test_try_except_node__with_as_syntax(self):
         source = dedent("""\
             try:
                 pass
@@ -1225,13 +1126,12 @@ class PatchedASTTest(unittest.TestCase):
             node_to_test,
             ["try", "", ":", "\n    ", "Pass", "\n", ("excepthandler", "ExceptHandler")],
         )
-        expected_child = "e" 
+        expected_child = "e"
         checker.check_children(
             ("excepthandler", "ExceptHandler"),
             ["except", " ", "Name", " ", "as", " ", expected_child, "", ":", "\n    ", "Pass"],
         )
 
-    @testutils.only_for("2.5")
     def test_try_except_and_finally_node(self):
         source = dedent("""\
             try:
@@ -1266,18 +1166,10 @@ class PatchedASTTest(unittest.TestCase):
         checker = _ResultChecker(self, ast_frag)
         checker.check_children("Module", ["", "Expr", "\n", "Expr", "\n"])
 
-    @testutils.only_for_versions_lower("3")
-    def test_how_to_handle_old_not_equals(self):
-        source = "1 <> 2\n"
-        ast_frag = patchedast.get_patched_ast(source, True)
-        checker = _ResultChecker(self, ast_frag)
-        checker.check_children("Compare", ["Num", " ", "<>", " ", "Num"])
-
     def test_semicolon(self):
         source = "1;\n"
         patchedast.get_patched_ast(source, True)
 
-    @testutils.only_for("2.5")
     def test_if_exp_node(self):
         source = "1 if True else 2\n"
         ast_frag = patchedast.get_patched_ast(source, True)
@@ -1322,7 +1214,6 @@ class PatchedASTTest(unittest.TestCase):
             ["Name", "", "(", "", "keyword", "", ",", " ", "*", "", "Name", "", ")"],
         )
 
-    @testutils.only_for("3.5")
     def test_starargs_before_keywords(self):
         source = "foo(*args, a=1)\n"
         ast_frag = patchedast.get_patched_ast(source, True)
@@ -1331,7 +1222,6 @@ class PatchedASTTest(unittest.TestCase):
             "Call", ["Name", "", "(", "*", "Starred", "", ",", " ", "keyword", "", ")"]
         )
 
-    @testutils.only_for("3.5")
     def test_starargs_in_keywords(self):
         source = "foo(a=1, *args, b=2)\n"
         ast_frag = patchedast.get_patched_ast(source, True)
@@ -1341,7 +1231,15 @@ class PatchedASTTest(unittest.TestCase):
             ["Name", "", "(", "", "keyword", "", ",", " *", "Starred", "", ",", " ", "keyword", "", ")"],
         )
 
-    @testutils.only_for("3.5")
+    def test_starargs_in_positional(self):
+        source = "foo(a, *b, c)\n"
+        ast_frag = patchedast.get_patched_ast(source, True)
+        checker = _ResultChecker(self, ast_frag)
+        checker.check_children(
+            "Call",
+            ["Name", "", "(", "", "Name", "", ",", " *", "Starred", "", ",", " ", "Name", "", ")"],
+        )
+
     def test_starargs_after_keywords(self):
         source = "foo(a=1, *args)\n"
         ast_frag = patchedast.get_patched_ast(source, True)
@@ -1371,7 +1269,7 @@ class PatchedASTTest(unittest.TestCase):
         checker = _ResultChecker(self, ast_frag)
         self.assert_single_case_match_block(checker, "MatchValue")
         checker.check_children("MatchValue", [
-            "Constant"
+            "Num"
         ])
 
     @testutils.only_for_versions_higher("3.10")
@@ -1426,7 +1324,7 @@ class PatchedASTTest(unittest.TestCase):
             ")",
         ])
         checker.check_children("MatchValue", [
-            "Constant"
+            "Num"
         ])
 
     @testutils.only_for_versions_higher("3.10")
@@ -1554,6 +1452,35 @@ class PatchedASTTest(unittest.TestCase):
             ")",
         ])
 
+    @testutils.only_for_versions_higher("3.10")
+    def test_match_node_with_match_mapping_match_as(self):
+        source = dedent("""\
+            match x:
+                case {"a": b} as c:
+                    print(x)
+        """)
+        ast_frag = patchedast.get_patched_ast(source, True)
+        checker = _ResultChecker(self, ast_frag)
+        self.assert_single_case_match_block(checker, "MatchAs")
+        checker.check_children("MatchAs", [
+            "MatchMapping",
+            " ",
+            "as",
+            " ",
+            "c",
+        ])
+        checker.check_children("MatchMapping", [
+            "{",
+            "",
+            "Str",
+            "",
+            ":",
+            " ",
+            "MatchAs",
+            "",
+            "}",
+        ])
+
 
 class _ResultChecker:
     def __init__(self, test_case, ast):
@@ -1567,6 +1494,32 @@ class _ResultChecker:
         self.test_case.assertEqual((start, end), node.region)
 
     def _find_node(self, text):
+        """
+        Find the node in `self.ast` whose type is named in `text`.
+
+        :param text: ast node name
+
+        Generally, the test should only have a single matching node, as it make
+        the test much harder to understand when there may be multiple matches.
+
+        If `self.ast` contains more than one nodes that matches `text`, then
+        the **outer-most last match** takes precedence.
+
+        For example, given that we are looking for `ast.Call` node:
+
+            checker._find_node("Call")
+
+        and given that `self.ast` is the AST for this code:
+
+            func_a(1, func_b(2, 3)) + func_c(4, func_d(5, 6))
+
+        the outer-most last match would be the ast node representing this bit:
+
+            func_c(4, func_d(5, 6))
+
+        Note that the order of traversal is based on the order of ast nodes,
+        which usually, but not always, match textual order.
+        """
         goal = text
         if not isinstance(text, (tuple, list)):
             goal = [text]
@@ -1576,23 +1529,16 @@ class _ResultChecker:
 
             def __call__(self, node):
                 for text in goal:
-                    if sys.version_info >= (3, 8) and text in [
-                        "Num",
-                        "Str",
-                        "NameConstant",
-                        "Ellipsis",
-                    ]:
-                        text = "Constant"
                     if str(node).startswith(text):
                         self.result = node
                         break
-                    if node.__class__.__name__.startswith(text):
+                    if ast.get_node_type_name(node).startswith(text):
                         self.result = node
                         break
                 return self.result is not None
 
         search = Search()
-        ast.call_for_nodes(self.ast, search, recursive=True)
+        ast.call_for_nodes(self.ast, search)
         return search.result
 
     def check_children(self, text, children):
@@ -1606,20 +1552,13 @@ class _ResultChecker:
             if not isinstance(expected, (tuple, list)):
                 goals = [expected]
             for goal in goals:
-                if goal == "" or isinstance(child, basestring):
+                if goal == "" or isinstance(child, (str, bytes)):
                     self.test_case.assertEqual(goal, child)
                     break
             else:
                 self.test_case.assertNotEqual("", text, "probably ignoring some node")
-                if sys.version_info >= (3, 8) and expected in [
-                    "Num",
-                    "Str",
-                    "NameConstant",
-                    "Ellipsis",
-                ]:
-                    expected = "Constant"
                 self.test_case.assertTrue(
-                    child.__class__.__name__.startswith(expected),
+                    ast.get_node_type_name(child).startswith(expected),
                     msg="Expected <%s> but was <%s>"
-                    % (expected, child.__class__.__name__),
+                    % (expected, ast.get_node_type_name(child)),
                 )
