@@ -1,10 +1,6 @@
+import unittest
 from textwrap import dedent
 
-try:
-    import unittest2 as unittest
-except ImportError:
-    import unittest
-
 from rope.refactor import similarfinder
 from ropetest import testutils
 
